/**
 * Diese Klasse implementiert eine Ballsuchstrategie: Der Spieler
 * dreht sich solange, bis er den Ball sieht.
 * 
 * @author Sebastian Hantsch
 * @version 1.0
 */
public class BallFindAction extends BasicAction
{
	/** Zeitpunkt des letzten turn-Befehls */
	private int lastTurn;

	/**
	 * Erzeugt eine neue <code>BallFindAction</code>
	 * @param worldModel Referenz zum Weltmodell
	 * @param sender Referenz zum Sender
	 */
	public BallFindAction(WorldModel worldModel, Sender sender)
	{
		super(worldModel, sender);

		lastTurn = 0;
	}

	/**
	 * Aktion fortsetzen
	 */
	public void resume()
	{
		// Ist der Ball zu sehen?
		if (worldModel.getActTime() - worldModel.getBall().getTime() < 8)
		{
			// Ja, also ist die Aktion fertig
			actionFinished = true;
			actionPending = false;
		}
		else
		{
			// der Ball ist noch nicht zu sehen, also drehen
			if (lastTurn == 0)
			{
				// falls der Kopf nicht nach vorne sieht
				// nach vorne drehen
				double headAngle = worldModel.getYourself().getHeadAngle();
				if (headAngle != 0)
				{
					sender.turnNeck(-headAngle);
				}
			}

			// Wenn seit dem letzten turn Befehl ausreichend Zeit
			// verstrichen ist, weiterdrehen.
			if (lastTurn < worldModel.getYourself().getTime() - 2)
			{
				sender.turn(90);
				lastTurn = worldModel.getYourself().getTime();
			}
		}
	}
}
