/**
 * Diese Klasse reprsentiert eine Ballpositionsnachricht
 * eines Mitspielers.<br>
 * Sie enthlt sowohl Position als auch Bewegungsvektor des
 * Balles.
 * 
 * @author Sebastian Hantsch
 * @version 1.0
 */
public class BallPosMsg extends HearMsg
{
	/** Position des Balls */
	private Arrow ballPos;
	/** Bewegungsvektor des Balls */
	private Arrow ballVec;

	/**
	 * Erzeugt eine neue <code>BallPosMsg</code>.
	 */
	public BallPosMsg()
	{
		super();
	}

	/**
	 * Verarbeitung der Teilstcke der gehrten Nachricht.
	 * 
	 * @param ParenthesesTokenizer mTokens Teilstcke der Nachricht
	 * @return <code>true</code>, falls Nachricht gltiges Format aufweist,
	 * sonst <code>false</code>.
	 */
	public boolean processTokens(ParenthesesTokenizer mTokens)
	{
		// Zunchst processTokens der Superklasse ausfhren,
		// um die allgemeinen Nachrichtbestandteile auszuwerten.
		if (super.processTokens(mTokens))
		{
			// Da processTokens der Superklasse erfolgreich war,
			// knnen nun die speziellen Bestandteile ausgewertet
			// werden.			
			try
			{
				// Ballposition auslesen
				ballPos =
					new Arrow(
						Double.parseDouble(mTokens.nextToken()),
						Double.parseDouble(mTokens.nextToken()));
				// Ballvektor auslesen
				ballVec =
					new Arrow(
						Double.parseDouble(mTokens.nextToken()),
						Double.parseDouble(mTokens.nextToken()));
				// Wenn es bis hierher keine Exception gab,
				// war die Auswertung erfolgreich
				return true;
			}
			catch (NumberFormatException nfe)
			{
				// Eine Exception bedeutet ungltige Nachricht
				return false;
			}
		}
		else
		{
			// Schon bei der Superklasse ist die Auswertung
			// fehlgeschlagen
			return false;
		}
	}
	
	/**
	 * Gibt die Ballposition an.
	 * @return Arrow ballPos
	 */
	public Arrow getBallPos()
	{
		return ballPos;
	}

	/**
	 * Gibt den Ballvektor an.
	 * @return Arrow ballVec
	 */
	public Arrow getBallVec()
	{
		return ballVec;
	}
}
