/**
 * Diese Klasse ist die Superklasse fr alle Aktionen der
 * Strategie Jogging.
 * 
 * @author Sebastian Hantsch
 * @version 1.0
 */
public abstract class BasicAction
{
	/** Referenz zum Weltmodell */
	protected WorldModel worldModel;
	/** Referenz zum Sender */	
	protected Sender sender;
	/** Ist die Aktion beendet */
	protected boolean actionFinished;
	/** Ist die Aktion noch in Ausfhrung */
	protected boolean actionPending;	

	/**
	 * Erzeugt eine neue <code>BasicAction</code>
	 * @param worldModel Referenz zum Weltmodell
	 * @param sender Referenz zum Sender
	 */
	public BasicAction(WorldModel worldModel, Sender sender)
	{
		this.worldModel = worldModel;
		this.sender = sender;
		
		actionPending = false;
		actionFinished = false;
	}

	/**
	 * Aktion starten
	 */	
	public void start()
	{
		actionPending = true;
	}
	
	/**
	 * Aktion fortsetzen
	 */
	public abstract void resume();

	/**
	 * Gibt an, ob die Aktion beendet wurde.
	 * @return <code>true</code>, falls die Aktion beendet wurde,
	 * sonst <code>false</code>.
	 */
	public boolean isActionFinished()
	{
		return actionFinished;
	}

	/**
	 * Gibt an, ob die Aktion noch luft.
	 * @return <code>true</code>, falls die Aktion noch luft,
	 * sonst <code>false</code>.
	 */
	public boolean isActionPending()
	{
		return actionPending;
	}
}
