/**
 * Diese Klasse implementiert das Verhalten eines Abwehrspielers.
 * 
 * @author Sebastian Hantsch
 * @version 1.1
 */
public class Defense extends BasicRating 
{
	/**
	 * Erzeugt eine neue <code>Defense</code>.
	 * @param worldModel Referenz zum Weltmodell
	 * @param sender Referenz zum Sender
	 */
	public Defense(WorldModel worldModel, Sender sender)
	{
		super(worldModel, sender);
	
		// Abhngig von der Spielernummer Positionen setzen	
		switch (worldModel.getYourself().getNumber())
		{
			case 2:
				preferredPos = new Arrow(-35, -15);
				startPos = new Arrow(-40, -2);
				break;
			case 3:
				preferredPos = new Arrow(-30, 0);
				startPos = new Arrow(-40, 0);
				break;
			case 4:
				preferredPos = new Arrow(-35, 15);
				startPos = new Arrow(-40, 2);								
				break;
		}		
	}
	
	/**
	 * Gibt an, ob eine Position relevant fr den Spieler ist.
	 * @param pos Position
	 */
	protected boolean isRelevantPos(Arrow pos)
	{
		// Nur Positionen in eigener Hlfte und
		// im entsprechenden Streifen relevant
		boolean inSector;
		switch (worldModel.getYourself().getNumber())
		{
			case 2: inSector = pos.getY() < -10; break;
			case 3: inSector = Math.abs(pos.getY()) < 15; break;
			case 4: inSector = pos.getY() > 10; break;
			default: inSector = false;
		}
		return pos.getX() < 0 && inSector;
	}
	
	/**
	 * Aktuelle Spielsituation beurteilen und entsprechend
	 * neue Spielaktion bestimmen.
	 * 
	 * @param curAction aktuelle Aktion
	 * @return neue Aktion
	 */
	public BasicAction rate(BasicAction curAction)
	{
		int ballAge = worldModel.getActTime() - worldModel.getBall().getTime();
		Arrow myPos = worldModel.getYourself().getPosition();
		Arrow ballPos = worldModel.getBall().getPosition();	
	
		// Spielunterbrechung?	
		if (isGameBreak())
		{
			// Ja, also zur Startposition bewegen
			if (!moved)
			{
				if (RoboCupMath.distance(myPos, startPos) > 0.5)
				{
					MoveAction ma = new MoveAction(worldModel, sender);
					ma.setDestination(startPos);
					ma.start();
					return ma;
				}
				else if (Math.abs(worldModel.getYourself().getViewAngle()) > 5)
				{
					TurnAction ta = new TurnAction(worldModel, sender);
					ta.setDestWinkel(0);
					ta.start();
					return ta;
				}
				else
				{
					moved = true;
				}					
			}
		}
		else if (isEnemyFreeKick())
		{
			// Gegner hat Freistoss, also nichts besseres
			// zu tun, als den Ball zu suchen
			BallFindAction bfa = getBallFindAction(curAction);
			bfa.start();
			return bfa;			
		}		
		else
		{
			moved = false;
			// Ball sichtbar?
			if (ballAge < 5 && !isOwnGoalKick())
			{
				reportBallPos();
			
				// Ball in Schureichweite?
				double ballDiff = RoboCupMath.distance(worldModel.getYourself().getPosition(), worldModel.getBall().getPosition());
				
				if (ballDiff < 1)
				{
					// In Schussreichweite, also Ziel suchen
					// und schiessen
					KickAction ka = new KickAction(worldModel, sender);
					Arrow defDest;
					if (myPos.getY() < 0)
					{
						defDest = new Arrow(0, -20);
					}
					else
					{
						defDest = new Arrow(0, 20);
					}
					ka.setDestination(getKickTarget(myPos, defDest));
					ka.start();
					return ka;
				}
				else if (ballDiff < 10)
				{
					// Ball nahe, also hinrennen
					RunAction ra = getRunAction(curAction);
					ra.setDestination(getBallCollCourse());
					ra.start();
					return ra;
				}
				else	
				{	
					// Ball in eigener Hlfte?
					if (isRelevantPos(ballPos))
					{
						// Ja, also hinrennen
						RunAction ra = getRunAction(curAction);
						ra.setDestination(getBallCollCourse());
						ra.start();
						return ra;
					}
					else
					{
						// nein, also zur bevorzugten Position
						// rennen
						RunAction ra = getRunAction(curAction);
						ra.setDestination(preferredPos);
						ra.start();
						return ra;
					}
				}
			}
			else
			{
				if (ballAge > 15 || (ballAge > 20 && !worldModel.isballSeen()))
				{
					// Ball suchen
					BallFindAction bfa = getBallFindAction(curAction);
					bfa.start();
					return bfa;
				}
				else
				{
					// zur bevorzugten Position laufen
					RunAction ra = getRunAction(curAction);
					ra.setDestination(preferredPos);
					ra.start();
					return ra;
				}
			}					
		}
		
		return curAction;
	}   
}
