/**
 * Diese Klasse implementiert das Verhalten eines Strmers.
 * 
 * @author Sebastian Hantsch
 * @version 1.1
 */
public class Forward extends BasicRating
{
	/**
	 * Erzeugt einen neuen <code>Forward</code>.
	 * @param worldModel Referenz zum Weltmodell
	 * @param sender Referenz zum Sender
	 */
	public Forward(WorldModel worldModel, Sender sender)
	{
		super(worldModel, sender);

		switch (worldModel.getYourself().getNumber())
		{
			case 9 :
				preferredPos = new Arrow(-2, -15);
				startPos = new Arrow(-10, -15);
				break;
			case 10 :
				preferredPos = new Arrow(-2, 0);
				startPos = new Arrow(-5, 0);
				break;
			case 11 :
				preferredPos = new Arrow(-2, 15);
				startPos = new Arrow(-10, 15);
				break;
		}
	}

	/**
	 * Gibt an, ob eine Position relevant fr den Spieler ist.
	 * @param pos Position
	 */
	protected boolean isRelevantPos(Arrow pos)
	{
		boolean inSector = false;
		switch (worldModel.getYourself().getNumber())
		{
			case 9:
				if (pos.getY() < -10)
					inSector = true;
				break;
			case 10:
				if (Math.abs(pos.getY()) <= 10)
					inSector = true;
				break;
			case 11:
				if (pos.getY() > 10)
					inSector = true;
				break;
		}
		
		return pos.getX() > -5
			&& (Math.abs(pos.getY() - preferredPos.getY()) < 15 || inSector);
	}

	/**
	 * Aktuelle Spielsituation beurteilen und entsprechend
	 * neue Spielaktion bestimmen.
	 * 
	 * @param curAction aktuelle Aktion
	 * @return neue Aktion
	 */
	public BasicAction rate(BasicAction curAction)
	{
		int ballAge = worldModel.getActTime() - worldModel.getBall().getTime();
		Arrow myPos = worldModel.getYourself().getPosition();
		Arrow ballPos = worldModel.getBall().getPosition();

		if (isGameBreak())
		{
			if (!moved)
			{
				if (RoboCupMath.distance(myPos, startPos) > 0.5)
				{
					MoveAction ma = new MoveAction(worldModel, sender);
					ma.setDestination(startPos);
					ma.start();
					return ma;
				}
				else
				{
					TurnAction ta = new TurnAction(worldModel, sender);
					ta.setDestWinkel(0);
					ta.start();
					moved = true;
					return ta;
				}
			}
		}
		else if (isEnemyFreeKick())
		{
			BallFindAction bfa = getBallFindAction(curAction);
			bfa.start();
			return bfa;			
		}
		else
		{
			moved = false;
			if (ballAge < 5 && !isOwnGoalKick())
				reportBallPos();

			// Ball sichtbar?
			if (ballAge <= 15)
			{

				// Ball in Schureichweite?
				double ballDiff = RoboCupMath.distance(myPos, ballPos);

				if (ballDiff < 1)
				{
					KickAction ka = new KickAction(worldModel, sender);				
					if (RoboCupMath.distance(myPos, enemyGoal) < 20 ||
					    !worldModel.getMatchStatus().equals("play_on"))
					{
						ka.setPower(100);
					}
					else
					{
						int nearEnemies = 0;
						for (int i = 1; i <= worldModel.getEnemyCount(); i++)
						{
							if (worldModel.getEnemy(i) != null)
							{
								if (RoboCupMath.distance(myPos, worldModel.getEnemy(i).getPosition()) < 8 &&
								    worldModel.getEnemy(i).getPosition().getX() > myPos.getX())
								{
									nearEnemies++;
								}
							}
						}
						
						if (nearEnemies > 1)
						{
							ka.setPower(100);
						}
						else
						{
							ka.setPower(23);
						}
					}
					ka.setDestination(enemyGoal);
					ka.start();
					return ka;
				}
				else if (ballDiff < 10)
				{
					RunAction ra = getRunAction(curAction);
					ra.setDestination(getBallCollCourse());
					ra.start();
					return ra;
				}
				else
				{
					// Ball im gegnerischen Gebiet?
					if (isRelevantPos(ballPos) && worldModel.getYourself().getStamina() > 2500)
					{
						RunAction ra = getRunAction(curAction);
						ra.setDestination(getBallCollCourse());
						ra.start();
						return ra;
					}
					else
					{
						RunAction ra = getRunAction(curAction);
						if (!isOffside())
						{
							ra.setDestination(preferredPos);
						}
						else
						{
							ra.setDestination(startPos);
						}
						ra.start();
						return ra;
					}
				}
			}
			else
			{
				if (ballAge > 15)
				{
					// Ball suchen
					BallFindAction bfa = getBallFindAction(curAction);
					bfa.start();
					return bfa;
				}
				else
				{
					RunAction ra = getRunAction(curAction);
					if (!isOffside())
					{
						ra.setDestination(preferredPos);
					}
					else
					{
						ra.setDestination(startPos);
					}
					ra.start();
					return ra;
				}
			}
		}
		return curAction;
	}
}
