/**
 * Diese Klasse implementiert das Verhalten eines Torwarts.
 * 
 * @author Sebastian Hantsch
 * @version 1.1
 */
public class Goalie extends BasicRating
{
	private int lastCatch;
	
	/**
	 * Erzeugt einen neuen <code>Goalie</code>.
	 * @param worldModel Referenz zum Weltmodell
	 * @param sender Referenz zum Sender
	 */
	public Goalie(WorldModel worldModel, Sender sender)
	{
		super(worldModel, sender);

		// Positionen setzen	
		preferredPos = new Arrow(-50, 0);
		startPos = new Arrow(-50, 0);
		lastCatch = 0;
	}

	/**
	 * Gibt an, ob eine Position relevant fr den Spieler ist.
	 * @param pos Position
	 */
	protected boolean isRelevantPos(Arrow pos)
	{
		return pos.getX() < -20;
	}

	/**
	 * Bestimmt eine Position zum Abfangen des Balls
	 * @return Abfangposition
	 */
	protected Arrow getBallCatchCourse()
	{
		Arrow bPos = worldModel.getBall().getPosition();
		Arrow bVec = worldModel.getBall().getVector();
		Arrow futurePos = new Arrow(bPos.getX(), bPos.getY());

		// Luft der Ball Richtung Tor?
		if (bVec.getX() < 0)
		{
			if (bPos.getX() < -47.5 && Math.abs(bPos.getY()) < 15)
			{
				return bPos;
			}
			else
			{
				do
				{
					futurePos.addArrow(bVec);
				}
				while (futurePos.getX() > -47.5);

				if (Math.abs(futurePos.getY()) < 15)
				{
					return futurePos;
				}
				else
				{
					if (futurePos.getY() > 0)
					{
						return new Arrow(-48, 10);
					}
					else
					{
						return new Arrow(-48, -10);
					}
				}
			}
		}
		else
		{
			return preferredPos;
		}
	}

	/**
	 * Aktuelle Spielsituation beurteilen und entsprechend
	 * neue Spielaktion bestimmen.
	 * 
	 * @param curAction aktuelle Aktion
	 * @return neue Aktion
	 */
	public BasicAction rate(BasicAction curAction)
	{
		int ballAge = worldModel.getActTime() - worldModel.getBall().getTime();
		Arrow myPos = worldModel.getYourself().getPosition();
		Arrow ballPos = worldModel.getBall().getPosition();
		double bodyAngle = worldModel.getYourself().getViewAngle() - worldModel.getYourself().getHeadAngle();

		if (isGameBreak())
		{
			if (!moved)
			{
				if (RoboCupMath.distance(myPos, startPos) > 0.5)
				{
					MoveAction ma = new MoveAction(worldModel, sender);
					ma.setDestination(startPos);
					ma.start();
					return ma;
				}
				else
				{
					TurnAction ta = new TurnAction(worldModel, sender);
					ta.setResetHeadAngle(true);
					ta.setDestWinkel(0);
					ta.start();
					moved = true;
					return ta;
				}
			}
		}
		else if (isEnemyFreeKick())
		{
			BallFindAction bfa = getBallFindAction(curAction);
			bfa.start();
			return bfa;
		}
		else
		{
			moved = false;
			if (isOwnGoalKick())
			{
				if (worldModel.isballSeen())
				{
					Arrow kickPos = new Arrow(ballPos.getX(), ballPos.getY());
					kickPos.setX(kickPos.getX() - 0.5);

					if (RoboCupMath.distance(myPos, kickPos) > 0.5)
					{
						RunAction ra = getRunAction(curAction);
						ra.setDestination(kickPos);
						ra.start();
						return ra;
					}

					if (Math.abs(bodyAngle) > 5)
					{
						TurnAction ta = getTurnAction(curAction);
						ta.setResetHeadAngle(true);
						ta.setDestWinkel(0);
						ta.start();
						return ta;
					}

					KickAction ka = new KickAction(worldModel, sender);
					ka.setDestination(getKickTarget(myPos, new Arrow(0, -20)));
					ka.start();
					return ka;
				}
				else
				{
					BallFindAction bfa = getBallFindAction(curAction);
					bfa.start();
					return bfa;
				}
			}
			else if (isOwnFreeKick())
			{
			   	if (Math.abs(bodyAngle) > 5)
				{
				    TurnAction ta = getTurnAction(curAction);
				    ta.setResetHeadAngle(true);
				    ta.setDestWinkel(0);
				    ta.start();
				    return ta;
				}
				else
				{
				    KickAction ka = new KickAction(worldModel, sender);
				    ka.setDestination(getKickTarget(myPos, new Arrow(0, 0)));
				    ka.start();
				    return ka;
				}
			}
			else
			{
				// Ball sichtbar?
				if (ballAge < 5)
				{
					reportBallPos();

					if (isRelevantPos(ballPos))
					{
						// Ball in Fangreichweite?
						if (RoboCupMath.distance(myPos, ballPos) < 2)
						{
							if (worldModel.getActTime() - lastCatch >= 5)
							{
								CatchAction ca =
									new CatchAction(worldModel, sender);
								ca.start();
								lastCatch = worldModel.getActTime();
								return ca;
							}
							else
							{
							    KickAction ka = new KickAction(worldModel, sender);
							    ka.setDestination(getKickTarget(myPos, new Arrow(0, 0)));
							    ka.start();
							    return ka;								
							}
						}
						else
						{							
							if (ballPos.getX() > myPos.getX())
							{
								Arrow ballCatchPos = getBallCatchCourse();								
								Arrow ballVec = worldModel.getBall().getVector();
								double ballSpeed = Math.sqrt(ballVec.getX() * ballVec.getX() + ballVec.getY() * ballVec.getY());
								
								if (ballPos.getX() < -36.5)
								{
									if (ballSpeed < 0.4)
									{
										RunAction ra = getRunAction(curAction);
										ra.setDestination(getBallCollCourse());
										ra.start();
										return ra;
									}
								}								
								GoalieSlideAction gsa =
									getGoalieSlideAction(curAction);
								gsa.setDestination(ballCatchPos);
								gsa.start();
								return gsa;
							}
							else
							{
								RunAction ra = getRunAction(curAction);							
								ra.setDestination(ballPos);
								ra.start();
								return ra;
							}
						}
					}
					else
					{
						if (RoboCupMath.distance(myPos, preferredPos) > 1)
						{
							RunAction ra = getRunAction(curAction);
							ra.setDestination(preferredPos);
							ra.start();
							return ra;
						}
						else
						{
							TurnAction ta = getTurnAction(curAction);
							ta.setResetHeadAngle(true);
							ta.setDestWinkel(0);
							ta.start();
							return ta;
						}
					}
				}

				else
				{
					if (ballAge > 10)
					{
						// Ball suchen
						BallFindAction bfa = getBallFindAction(curAction);
						bfa.start();
						return bfa;
					}
					else
					{
						RunAction ra = getRunAction(curAction);
						ra.setDestination(preferredPos);
						ra.start();
						return ra;
					}
				}
			}
		}
		return curAction;
	}
}
