/**
 * Klasse fr die Bewertung des Torwarts
 * 
 * @author Jrg Zentgraf
 * @version 1.0
 */

public class GoalieRating extends Rating 
{
	/**Hat sich der Torwart nach dem Fangen schon bewegt ?*/
	private boolean movedFreeKick;
   
   	/**
   	 * Konstruktor
   	 * @param worldModel Weltmodell
   	 * @param movement Movement
   	 * @param sender Sender
   	 */
	public GoalieRating(WorldModel worldModel, Movement movement,Sender sender)
 	{
 		super(worldModel,movement,sender);
   		counter =0;
   		movedFreeKick=false;

   	}
   
	/**
	* Fhrt die Bewertung fr den Torwart aus <br>
    * Ist der Ball nah, soll er gefangen werden <br>  
    * Ist der Ball im Strafraum rennt der Torwart auf
    * die Ball Collision Position <br>
    * Ist der Ball in der eigenen Hlfte rennt der Torwart
    * auf die Ball Collision Position mit X-Koordinate -48 <br>
    * Ist der Ball in der gegnerischen Hlfte rennt  der Torwart
    * auf die Default-Position von -48,0
    */
	public void execute()
	{
		
		hero = worldModel.getYourself();
		ball = worldModel.getBall();
		int ballAge = worldModel.getActTime() - worldModel.getBall().getTime();
		Arrow ballPos = ball.getPosition();
		Arrow myPos = hero.getPosition();
		Arrow preferredPos = Formation.getPreferredPos(1);
		checkRun();
		//Wenn du nicht auf dem Spielfeld ist,
		//drehe dich, bis du Linien siehst,
		//und renne wieder ins Spielfeld
		if (isGameBreak())
		{
			//Startaustellung und StartTurn
	   		Arrow startPos=Formation.getStartPosition(1,kickoff(),
	   					worldModel.getOurScore(),worldModel.getPlayDirection());
			if (RoboCupMath.distance(myPos, startPos) > 0.5)
				movement.move(startPos);
			else
				startTurn();
		}
		else
		{
			if (!worldModel.isOnField())
			{
				if (worldModel.isLineVisible())
					movement.dash(80);			
				else
					if (counter%2==0)
						movement.turn(60);
			}
			else
			{
				/* Fhre Abstoss aus
					indem du in die Nhe des Balles rennst und den dann zum
					Aussenverteidiger spielst*/
				if (isOwnGoalKick())
				{
					Arrow kickPos = new Arrow(ballPos.getX(), ballPos.getY());
					kickPos.setX(kickPos.getX() - 0.5);
					if (ballAge<3)
					{
						if (RoboCupMath.distance(myPos, kickPos) > 0.5)
							movement.run(kickPos);
						else
						{
							if (myPos.getY()<0)
								movement.kickToPos(new Arrow(-30,-20));
							else
								movement.kickToPos(new Arrow(-30,20));
						}
					}
					else
						searchBall();
				}
				else if (isOwnGoalieFreeKick())
				{
					//Wenn der Ball gerade gefangen ist, dann bewege dich auf 
					//eine Position auf der anderen Spielhlfte (bzgl.y)
					//und spiele den Ball 
					if (!movedFreeKick)
						if (myPos.getY()<0)
						{
							movement.move(new Arrow(-40,5));
							movedFreeKick=true;
						}
						else
						{
							movement.move(new Arrow(-40,-5));
							movedFreeKick=true;
						}
					else
					{
						movement.kickToPos(getKickTarget());
						movedFreeKick=false;
					}
				}
				else
				{
					// Ball vor kurzem gesehen?
					if (ballAge < 5)
					{
						Arrow collCourseGoalie = getBallCollCourseGoalie();

						// Ball im Strafraum?
						if (penalty())
						{
							if (RoboCupMath.distance(ballPos,myPos)<0.7)
								movement.doCatch();
							else	
								movement.run(getBallCollCourse());
						}		
						else	
						{
							//Ball in eigener Hlfte ?
							if (isRelevantPos())
							{
								if (RoboCupMath.distance(collCourseGoalie,myPos)<0.6 && counter%3==0)
									movement.turn(ballPos);
								else
									movement.run(collCourseGoalie);
							}
							else
								if (RoboCupMath.distance(myPos, preferredPos)> 1)
									movement.run(preferredPos);
						}
					}

					else
					{
						if (ballAge > 10)
							// Ball suchen
							searchBall();
						else
							movement.run(preferredPos);
					}
				}

			}
		}		
		counter++;
	}
   
   
}
