/**
 * Diese Klasse implementiert das seitliche Laufverhalten
 * des Torwarts.
 * 
 * @author Sebastian Hantsch
 * @version 1.0
 */
public class GoalieSlideAction extends BasicAction
{
	/** Laufziel */
	private Arrow destination;
	/** letzter Aufruf des turn Befehls */
	private int lastTurn;
	/** letzter Aufruf des dash Befehls */
	private int lastDash;

	/**
	 * Neue <code>GoalieSlideAction</code> erzeugen.
	 * @param worldModel Referenz zum Weltmodell
	 * @param sender Referenz zum Sender
	 */
	public GoalieSlideAction(WorldModel worldModel, Sender sender)
	{
		super(worldModel, sender);
		destination = new Arrow();
		lastTurn = 0;
		lastDash = 0;
	}

	/**
	 * Aktion fortsetzen
	 */
	public void resume()
	{
		Arrow myPos = worldModel.getYourself().getPosition();
		double headAngle = worldModel.getYourself().getHeadAngle();
		double bodyAngle = worldModel.getYourself().getViewAngle() - headAngle;
		double destAngle;
		double destHeadAngle;
		double distToDest = Math.abs(myPos.getY() - destination.getY());
		boolean turnAllowed =
			Math.abs(worldModel.getBall().getPosition().getX() - myPos.getX())
				> 10;
		boolean runBackward = false;

		// Ist der Torwart schon am Ziel?
		if (distToDest < 1)
		{
			// Ja, dann nach vorne drehen
			destAngle = 0;
			destHeadAngle = 0;
		}
		else
		{
			// Nein, also entweder nach links oder nach rechts drehen
			if (destination.getY() > myPos.getY())
			{
				destAngle = 90.0;
				destHeadAngle = -90.0;
			}
			else
			{
				destAngle = -90.0;
				destHeadAngle = 90.0;
			}

			if (!turnAllowed)
			{
				if (destAngle * bodyAngle < 0)
				{
					runBackward = true;
					destAngle = -destAngle;
					destHeadAngle = -destAngle;
				}
			}
		}

		double angle = RoboCupMath.angleDiff(bodyAngle, destAngle);

		// ggf. den Kopfwinkel anpassen
		if (Math.abs(headAngle - destHeadAngle) > 5)
		{
			sender.turnNeck(destHeadAngle - headAngle);
		}

		if (Math.abs(angle) > 10)
		{
			// Krperwinkel korrigieren, falls gengend Zeit
			// seit letztem turn verstrichen
			if (lastTurn < worldModel.getActTime() - 2)
			{
				sender.turn(angle);
				lastTurn = worldModel.getActTime();
			}
		}
		else
		{
			// Winkel in Ordnung, also dash ausfhren
			if (distToDest > 1)
			{
				if (lastDash < worldModel.getActTime() - 1)
				{
					if (runBackward)
					{
						sender.dash(-100);
					}
					else
					{
						sender.dash(100);
					}
					lastDash = worldModel.getActTime();
				}
			}
			else
			{
				// Am Ziel, Winkel in Ordnung, also Aktion beenden
				actionFinished = true;
				actionPending = false;
			}
		}
	}

	/**
	 * Ziel der <code>GoalieSlideAction</code> setzen.
	 * @param destination das Ziel
	 */
	public void setDestination(Arrow destination)
	{
		this.destination = destination;
	}
}
