/**
 * Umsetzung der hear-Nachricht als Klasse.
 * 
 * 03.6.2002: processTokens eingefgt
 * 
 * @author Sebastian Hantsch
 * @version 1.2
 */
public class HearEvent extends TimeStampedEvent
{
	private String sender;
	private double direction;
	private String message;

	/**
	 * Erzeugt ein neues HearEvent.
	 */
	public HearEvent()
	{
		super();
		sender = "";
		message = "";
	}

	/**
	 * Liest die Teilstcke von tokens aus und belegt die
	 * Ereignisattribute entsprechend
	 * @param tokens Teilstcke
	 * @return <code>true</code>, falls gltige Anzahl/Art von 
	 * Token, sonst <code>false</code>
	 */
	public boolean processTokens(ParenthesesTokenizer tokens)
	{
		if (super.processTokens(tokens))
		{
			// gibt es berhaupt noch einen weiteren Token?
			if (tokens.hasMoreTokens())
			{
				sender = tokens.nextToken();
				// Wirklich ein Sender oder vielleicht eine Richtung?
				if (!sender.equals("online_coach_left")
					&& !sender.equals("online_coach_right")
					&& !sender.equals("referee")
					&& !sender.equals("self"))
				{
					direction = Double.parseDouble(sender);
					sender = "";
				}
				else
				{
					direction = 0.0;
				}

				if (tokens.hasMoreTokens())
				{
					message = tokens.nextToken();
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Gibt den Sender an.
	 * Falls in der Nachricht eine Richtung enthalten ist, ist
	 * sender ein leerer String.
	 * @return sender
	 */
	public String getSender()
	{
		return sender;
	}

	/**
	 * Gibt die Richtung an.
	 * @return <code>double</code> - Wert, der die Richtung enthlt. Dieser Wert ist 0, falls
	 * ein Sender in der Nachricht enthalten war.
	 */
	public double getDirection()
	{
		return direction;
	}

	/**
	 * Gibt die Nachricht an.
	 * @return Einen <code>String</code>, der die Nachricht in Hochkomma eingeschlossen enthlt.
	 */
	public String getMessage()
	{
		return message;
	}
}