/**
 * Diese abstrakte Klasse ist die Superklasse aller
 * Nachrichtenklassen. Sie wertet die allgemeinen Bestandteile
 * einer Nachricht aus.
 * 
 * @author Sebastian Hantsch
 * @version 1.0
 */
public abstract class HearMsg
{
	/** Spielernummer des sendenden Spielers */
	private int playerNumber;
	/** Nachrichtennummer des sendenden Spielers */	
	private int msgNumber;
	/** Sendezeit der Nachricht */	
	private int msgTime;
	/** Nachrichtentyp */
	private String msgType;
	
	/**
	 * Erzeugt eine neue Nachricht.
	 */
	public HearMsg()
	{
		msgType = "";
	}
	
	/**
	 * Verarbeitet die allgemeinen Bestandteile der Nachricht
	 * 
	 * @param ParenthesesTokenizer mTokens Teilstcke der Nachricht
	 * @return <code>true</code>, falls Nachricht erfolgreich ausgewertet
	 * wurde, sonst <code>false</code>.
	 */
	public boolean processTokens(ParenthesesTokenizer mTokens)
	{
		// teamHash berspringen
		mTokens.nextToken();
		try
		{
			// allgemeine Bestandteile auswerten
			playerNumber = Integer.parseInt(mTokens.nextToken());
			msgNumber = Integer.parseInt(mTokens.nextToken());
			msgTime = Integer.parseInt(mTokens.nextToken());
			msgType = mTokens.nextToken();
			// wenn es bis hierher keine Exception gab, war
			// die Auswertung erfolgreich
			return true;
		}
		catch (NumberFormatException nfe)
		{
			// Exception, also fehlerhafte Nachricht
			return false;
		}		
	}
		
	/**
	 * Gibt die Nachrichtennummer an.
	 * @return int msgNumber
	 */
	public int getMsgNumber()
	{
		return msgNumber;
	}

	/**
	 * Gibt die Sendezeit an.
	 * @return int msgTime
	 */
	public int getMsgTime()
	{
		return msgTime;
	}

	/**
	 * Gibt den Nachrichtentyp an.
	 * @return String msgType
	 */
	public String getMsgType()
	{
		return msgType;
	}

	/**
	 * Gibt die Spielernummer des Senders an.
	 * @return int playerNumber
	 */
	public int getPlayerNumber()
	{
		return playerNumber;
	}
}
