/**
 * Umsetzung der init-Nachricht als Klasse.
 * 
 * 27.5.2002: Abfrage auf ausreichend Tokens eingebaut.
 * 03.6.2002: processTokens eingefgt
 * 
 * @author Sebastian Hantsch
 * @version 1.2
 */
public class InitEvent extends ServerEvent
{
	private char side;
	private int unum;
	private String playMode;

	/**
	 * Erzeugt ein neues InitEvent.
	 */
	public InitEvent()
	{
		super();
		playMode = "";
	}

	/**
	 * Liest die Teilstcke von tokens aus und belegt die
	 * Ereignisattribute entsprechend
	 * @param tokens Teilstcke
	 * @return <code>true</code>, falls gltige Anzahl/Art von 
	 * Token, sonst <code>false</code>
	 */
	public boolean processTokens(ParenthesesTokenizer tokens)
	{
		if (super.processTokens(tokens))
		{
			if (tokens.tokenCount() >= 4)
			{
				side = tokens.nextToken().charAt(0);
				unum = Integer.parseInt(tokens.nextToken());
				playMode = tokens.nextToken();
				return true;
			}
		}
		return false;
	}

	/**
	 * Gibt die Seite an (l oder r).
	 * @return side
	 */
	public char getSide()
	{
		return side;
	}

	/**
	 * Gibt die Spielernummer an.
	 * @return unum
	 */
	public int getUnum()
	{
		return unum;
	}

	/**
	 * Gibt den Spielmodus an.
	 * @return unum
	 */
	public String getPlayMode()
	{
		return playMode;
	}
}