/**
 * Diese Klasse implementiert die Jogging Strategie.
 * 
 * @author Sebastian Hantsch
 * @version 1.0
 */
public class Jogging extends Strategy
{
	/** Beurteilungsobjekt des Spielers */
	private BasicRating rating;
	/** aktuelle Aktion des Spielers */
	private BasicAction curAction;
	/** Hrer des Spielers */
	private Hearer hearer;

	/**
	 * Neue <code>Jogging</code> erzeugen.
	 */
	public Jogging()
	{
		super();

		rating = null;
		curAction = null;
		hearer = null;
	}

	/**
	 * Diese Methode wird nach jeder empfangenen sense_body
	 * Nachricht aufgerufen.
	 */
	public void execute()
	{
		if (rating == null)
		{
			// Erster Aufruf von execute, also alle bentigten
			// Objekte erzeugen
			hearer = new Hearer(worldModel);
			// Abhngig von der Spielernummer verhalten zuweisen
			switch (worldModel.getYourself().getNumber())
			{
				case 1 :
					rating = new Goalie(worldModel, sender);
					break;
				case 2 :
				case 3 :
				case 4 :
					rating = new Defense(worldModel, sender);
					break;
				case 5 :
				case 6 :
				case 7 :
				case 8 :
					rating = new Midfield(worldModel, sender);
					break;
				case 9 :
				case 10 :
				case 11 :
					rating = new Forward(worldModel, sender);
					break;
				default:
					System.err.println("Keine gltige Spielernummer: " + worldModel.getYourself().getNumber());
					return;
			}
		}

		// Hears verarbeiten
		hearer.processHeard();
		BallPosMsg bpMsg = hearer.getBallPosMsg();
		if (bpMsg != null)
		{
			// Falls kein Ball sichtbar, dann gehrte
			// Ballposition eintragen
			if (!worldModel.isballSeen())
			{
				worldModel.getBall().setPosition(bpMsg.getBallPos());
				worldModel.getBall().setVector(bpMsg.getBallVec());
			}
		}
		
		// Ist Spieler auerhalb des Spielfelds?
		if (worldModel.getActTime() > 0 && !worldModel.isOnField())
		{
			// Ja, also ab zurck ins Feld
			rating.returnIntoField();
		}
		else
		{
			// Nein, also normale Situationsbewertung
			curAction = rating.rate(curAction);
			if (curAction != null)
			{
				// Aktion fortsetzen
				curAction.resume();
			}
		}
	}
}
