/**
 * Diese Klasse implementiert einen gezielten Schuss.
 * 
 * @author Sebastian Hantsch
 * @version 1.2
 */
public class KickAction extends BasicAction
{
	/** Schussziel */
	private Arrow destination;
	/** Schusskraft */
	private double power;
	/** Schusskraftvorgabe */
	private boolean powerSet;

	/**
	 * Erzeugt eine neue <code>KickAction</code>.
	 * @param worldModel Referenz zum Weltmodell.
	 * @param sender Referenz zum Sender.
	 */
	public KickAction(WorldModel worldModel, Sender sender)
	{
		super(worldModel, sender);
		destination = new Arrow();
		power = 100;
		powerSet = false;
	}
	
	/**
	 * Legt das Schussziel fest.
	 * @param destination das Schussziel
	 */
	public void setDestination(Arrow destination)
	{
		this.destination.setXY(destination.getX(), destination.getY());
	}
	
	/**
	 * Berechnet die ntige Schusskraft anhand der Entfernung
	 * zum Ball.
	 */
	private double calcKickPower()
	{
		// Entfernung zum Ball berechnen
		double dist = RoboCupMath.distance(worldModel.getYourself().getPosition(), destination);
		
		// abhngig von Entfernung die Schusskraft ermitteln
		if (dist < 40)
		{
			return 20.0 + 2 * dist;
		}
		else
		{
			return 100;
		}
	}

	/**
	 * Aktion fortsetzen.
	 */
	public void resume()
	{
		Hero me = worldModel.getYourself();
		if (RoboCupMath.distance(me.getPosition(), worldModel.getBall().getPosition())
			< 1.5)
		{
			// Im kickable Bereich			
			double targetAngle =
				RoboCupMath.angle(destination, me.getPosition());
			double diffAngle =
				RoboCupMath.angleDiff(
					worldModel.getYourself().getViewAngle(),
					targetAngle);
			if (powerSet)
			{
				sender.kick(power, diffAngle);
			}
			else
			{
				sender.kick(calcKickPower(), diffAngle);
			}
		}
		actionFinished = true;
		actionPending = false;
	}
	
	/**
	 * Schusskraft vorgeben.
	 * @param power die gewnschte Schusskraft
	 */
	public void setPower(double power) {
		this.power = power;
		powerSet = true;
	}
}
