/**
 * Diese Klasse implementiert das Verhalten eines Mittelfeldspielers.
 * 
 * @author Sebastian Hantsch
 * @version 1.1
 */
public class Midfield extends BasicRating
{
	/**
	 * Erzeugt eine neue <code>Defense</code>.
	 * @param worldModel Referenz zum Weltmodell
	 * @param sender Referenz zum Sender
	 */
	public Midfield(WorldModel worldModel, Sender sender)
	{
		super(worldModel, sender);

		switch (worldModel.getYourself().getNumber())
		{
			case 5 :
				preferredPos = new Arrow(-10, -20);
				startPos = new Arrow(-20, -3);
				break;
			case 6 :
				preferredPos = new Arrow(-10, -10);
				startPos = new Arrow(-20, -1);
				break;
			case 7 :
				preferredPos = new Arrow(-10, 10);
				startPos = new Arrow(-20, 1);
				break;
			case 8 :
				preferredPos = new Arrow(-10, 20);
				startPos = new Arrow(-20, 3);
				break;
		}
	}

	/**
	 * Gibt an, ob eine Position relevant fr den Spieler ist.
	 * @param pos Position
	 */
	protected boolean isRelevantPos(Arrow pos)
	{
		Arrow myPos = worldModel.getYourself().getPosition();

		if (myPos.getX() < 0)
		{
			return pos.getX() < 20
				&& pos.getX() > -30
				&& Math.abs(pos.getY() - preferredPos.getY()) < 15;
		}
		else
		{
			if (pos.getX() < myPos.getX())
			{
				return true;
			}
			else
			{
				return (RoboCupMath.distance(myPos, pos) < 20);
			}
		}
	}

	/**
	 * Aktuelle Spielsituation beurteilen und entsprechend
	 * neue Spielaktion bestimmen.
	 * 
	 * @param curAction aktuelle Aktion
	 * @return neue Aktion
	 */
	public BasicAction rate(BasicAction curAction)
	{
		int ballAge = worldModel.getActTime() - worldModel.getBall().getTime();
		Arrow myPos = worldModel.getYourself().getPosition();
		Arrow ballPos = worldModel.getBall().getPosition();

		if (isGameBreak())
		{
			if (!moved)
			{
				if (RoboCupMath.distance(myPos, startPos) > 0.5)
				{
					MoveAction ma = new MoveAction(worldModel, sender);
					ma.setDestination(startPos);
					ma.start();
					return ma;
				}
				else
				{
					TurnAction ta = new TurnAction(worldModel, sender);
					ta.setDestWinkel(0);
					ta.start();
					moved = true;
					return ta;
				}
			}
		}
		else if (isEnemyFreeKick())
		{
			BallFindAction bfa = getBallFindAction(curAction);
			bfa.start();
			return bfa;
		}
		else
		{
			moved = false;
			// Ball sichtbar?
			if (ballAge < 5 && !isOwnGoalKick())
			{
				reportBallPos();

				// Ball in Schureichweite?
				double ballDiff =
					RoboCupMath.distance(
						worldModel.getYourself().getPosition(),
						ballPos);

				if (ballDiff < 1)
				{
					KickAction ka = new KickAction(worldModel, sender);
					if (myPos.getX() > 10)
					{
						if (RoboCupMath.distance(myPos, enemyGoal) < 20
							|| !worldModel.getMatchStatus().equals("play_on"))
						{
							ka.setPower(100);
						}
						else
						{
							int nearEnemies = 0;
							for (int i = 1;
								i <= worldModel.getEnemyCount();
								i++)
							{
								if (worldModel.getEnemy(i) != null)
								{
									if (RoboCupMath
										.distance(
											myPos,
											worldModel
												.getEnemy(i)
												.getPosition())
										< 8
										&& worldModel
											.getEnemy(i)
											.getPosition()
											.getX()
											> myPos.getX())
									{
										nearEnemies++;
									}
								}
							}

							if (nearEnemies > 1)
							{
								ka.setPower(100);
							}
							else
							{
								ka.setPower(23);
							}
						}
						ka.setDestination(enemyGoal);
					}
					else
					{
						ka.setDestination(getKickTarget(myPos, enemyGoal));
					}
					ka.start();
					return ka;
				}
				else if (ballDiff < 10)
				{
					RunAction ra = getRunAction(curAction);
					ra.setDestination(getBallCollCourse());
					ra.start();
					return ra;
				}
				else
				{
					// Ball im Mittelfeld ?
					if (isRelevantPos(ballPos))
					{
						RunAction ra = getRunAction(curAction);
						ra.setDestination(getBallCollCourse());
						ra.start();
						return ra;
					}
					else
					{
						RunAction ra = getRunAction(curAction);
						ra.setDestination(preferredPos);
						ra.start();
						return ra;
					}
				}
			}
			else
			{
				if (ballAge > 15)
				{
					// Ball suchen
					BallFindAction bfa = getBallFindAction(curAction);
					bfa.start();
					return bfa;
				}
				else
				{
					RunAction ra = getRunAction(curAction);
					ra.setDestination(preferredPos);
					ra.start();
					curAction = ra;
				}
			}
		}
		return curAction;
	}
}
