/**
 * Diese Klasse enthlt die Grundbewegungen
 * 
 * @author Markus Rhein
 * @version 1.0
 */

public class Movement 
{
	/** Referenz auf Sender */
    public Sender sender;
    /** Referenz auf Weltmodell */
    private WorldModel worldModel;
   
    /** Zhlvariable */
    private int counter;
    /** Soll sich Spieler bewegen ? */
    private boolean shouldRun;


    /**
     * Konstruktor
     * @param sender
     * @param worldModel
     */
   	public Movement(Sender sender,WorldModel worldModel) 
   	{
   		this.worldModel=worldModel;
    	this.sender=sender;
    	counter=0;
   	}
   	
   	/**
   	 * Beschleunigt den Spieler
   	 * @param power
   	 */
   	public void dash(double power)
   	{
   		sender.dash(power);
   	}
   	
   	/**
   	 * Dreht den Spieler
   	 * @param angle
   	 */
   	public void turn(double angle)
   	{
   		sender.turn(angle);
   	}
   	
   	/**
   	 * Schiesst den Ball Richtung gegnerisches Tor
   	 */
   	public void kickToGoal()
   	{
   		kickToPos(new Arrow(52,0));
   	}
   		
	/**
	 * Schiesst den Ball in Richtung einer Position
	 * @param goesto
	 */   
	public void kickToPos(Arrow goesto)
	{
		/** Schustrke */
		double power;
		if (goesto.getX()==52)
			power=100;
		else 
			/** Schustrke in Abhngigkeit von der Zielentfernung */
			power=RoboCupMath.distance(goesto,worldModel.getYourself().getPosition())*4;
		if (power>100)
			power=100;
			
		/** Winkel zum Ziel von aktueller Position relativ zur x-Achse */	
  		double angle = RoboCupMath.angle(goesto, worldModel.getYourself().getPosition()); 
  		/** Winkel zum Ziel relativ zur eigenen Bewegungsrichtung */
  		double diffAngle = RoboCupMath.angleDiff(worldModel.getYourself().getViewAngle(), angle);
 		sender.kick(power,diffAngle);
   	}		   

   /**
    * Rennt zu einer Position
    * @param goesto
    */
    public void run(Arrow goesto)
    {
        /** Distanz zum Ziel */
        double dist = RoboCupMath.distance(worldModel.getYourself().getPosition(), goesto);
        if (shouldRun)
        {
	    	if (dist > 0.5)
 	        {            
  	       		/** Winkel zum Ziel von aktueller Position relativ zur x-Achse */
   	         	double angle = RoboCupMath.angle(goesto, worldModel.getYourself().getPosition());           
    	        /** Winkel zum Ziel relativ zur eigenen Bewegungsrichtung */
     	      	double diffAngle = RoboCupMath.angleDiff(worldModel.getYourself().getViewAngle(), angle);
      	     	// Bei groer Kursabweichung drehen, sonst laufen
       	 	    if (Math.abs(diffAngle) > 5)
        	    {
         	   		if (counter % 3 == 0)
                    	sender.turn(diffAngle);
                	else
                	{
                		if (Math.abs(diffAngle)<45)
	                		sender.dash(50);
                	}
                	counter++;
            	}
            	else
            	{
	           		counter=0;
	           		//Beschleunigt abhngig von der Zielentfernung
                	if (dist > 3)
                    	sender.dash(100);
                	else 
                    	sender.dash(50);
            	}
        	}
        }
	}

	/**
	 * leitet Catch weiter
	 * wird immer Richtung Ball ausgefhrt
	 */
	public void doCatch()
 	{
   		sender.doCatch(worldModel.getBall().getDirection());
    }
   
   	/**
   	 * leitet Move weiter
   	 * @param goesto
   	 */
	public void move(Arrow goesto)
    {
    	sender.move(goesto.getX(),goesto.getY());
    }
    
    /**
     * dreht sich zu einem Arrow
     * @param goesto
     */
    public void turn(Arrow goesto)
    {	
		/** Winkel zum Ziel von aktueller Position relativ zur x-Achse */
        double angle = RoboCupMath.angle(goesto, worldModel.getYourself().getPosition());           
        /** Winkel zum Ziel relativ zur eigenen Bewegungsrichtung */
        double diffAngle = RoboCupMath.angleDiff(worldModel.getYourself().getViewAngle(), angle);    	
        sender.turn(diffAngle);
    }        
    
	/**
	 * legt fest, ob sich der Spieler bewegen soll
	 * @param t
	 */
    public void setShouldRun(boolean t)
    {
    	shouldRun=t;
    }
    	
}
