import java.io.*;

/**
 * Diese Klasse dient zur Auswertung der Kommandozeilenparameter.
 * 
 * 12.06.2002: try/catch Blcke um Integer.parseInt, zustzliche Plausi<br>
 * 27.06.2002: Rechtschreibfehler in printSyntax korrigiert<br>
 * 
 * @author Sebastian Hantsch
 * @version 1.3
 */
public class ParameterParser
{
	/** Teamname des eigenen Teams */
	private String teamName;
	/** Serveradresse */
	private String serverName;
	/** Serverport */
	private int serverPort;
	/** Spieler, der zu Beginn visualisiert wird */
	private int visualize;
	/** Anzahl der zu erzeugenden Spieler */
	private int playerNum;
	/** Zu verwendende Strategieklasse */
	private String strategyClass;
	/** Kein Torwart erzeugen */
	private boolean noGoalie;
	/** Fehlerhafte Syntax der Kommandozeilenparameter */
	private boolean syntaxError;

	/**
	 * Gibt die korrekte Aufrufsyntax des Clients auf einem
	 * <code>PrintStream</code> aus.
	 * 
	 * @param PrintStream ps Der zu verwendende <code>PrintStream</code>.
	 */
	public static void printSyntax(PrintStream ps)
	{
		ps.println("Aufrufsyntax:");
		ps.println(
			"java RoboCup [teamname [servername [serverport]]] [optionen]");
		ps.println();
		ps.println("Mgliche Optionen (kombinierbar):");
		ps.println("-visualize num       Visualisierung fr den Spieler num");
		ps.println("-playernum num       Erzeuge num Spieler (Standard: 11)");
		ps.println("-nogoalie            Erster Spieler wird kein Torwart");
		ps.println(
			"-s strategy          Benutze Klasse strategy als Strategie (Standard: Strategy)");
	}

	/**
	 * Erzeugt einen neuen <code>ParameterParser</code>.
	 */
	public ParameterParser(String[] args)
	{
		// Standardbelegung setzen
		teamName = "namenlos";
		serverName = "localhost";
		serverPort = 6000;
		visualize = -1;
		playerNum = 11;
		strategyClass = "Strategy";
		syntaxError = false;
		noGoalie = false;

		// Kommandozeilenparameter auswerten
		boolean foundDashOpt = false;
		int i = 0;
		while (i < args.length)
		{
			if (args[i].startsWith("-"))
			{
				foundDashOpt = true;
				// "-" entfernen
				String optName = args[i].substring(1);

				if (optName.equals("visualize"))
				{
					// visualize erfordert eine Integerzahl als
					// nchsten Parameter, also diesen auswerten
					i++;
					if (i < args.length)
					{
						try
						{
							visualize = Integer.parseInt(args[i]);
						}
						catch (NumberFormatException nfe)
						{
							syntaxError = true;
						}
					}
					else
					{
						syntaxError = true;
					}
				}
				else if (optName.equals("playernum"))
				{
					// playernum erfordert eine Integerzahl als
					// nchsten Parameter, also diesen auswerten					
					i++;
					if (i < args.length)
					{
						try
						{
							playerNum = Integer.parseInt(args[i]);
						}
						catch (NumberFormatException nfe)
						{
							syntaxError = true;
						}
					}
					else
					{
						syntaxError = true;
					}
				}
				else if (optName.equals("s"))
				{
					// s erfordert einen String als
					// nchsten Parameter, also diesen auswerten					
					i++;
					if (i < args.length)
					{
						strategyClass = args[i];
					}
					else
					{
						syntaxError = true;
					}
				}
				else if (optName.equals("nogoalie"))
				{
					noGoalie = true;
				}
				else
				{
					syntaxError = true;
				}
			}
			else
			{
				if (!foundDashOpt)
				{
					switch (i)
					{
						case 0 :
							teamName = args[i];
							break;
						case 1 :
							serverName = args[i];
							break;
						case 2 :
							try
							{
								serverPort = Integer.parseInt(args[i]);
							}
							catch (NumberFormatException nfe)
							{
								syntaxError = true;
							}
							break;
						default :
							syntaxError = true;
							break;
					}
				}
				else
				{
					syntaxError = true;
				}
			}
			i++;
		}
		
		// Plausibilittsprfungen
		if (visualize > playerNum)
		{
			syntaxError = true;
		}
	}

	/**
	 * Gibt die Serveradresse an.
	 * @return String serverName
	 */
	public String getServerName()
	{
		return serverName;
	}

	/**
	 * Gibt den Serverport an.
	 * @return int serverPort
	 */
	public int getServerPort()
	{
		return serverPort;
	}

	/**
	 * Gibt den Spieler an, der zu Beginn visualisiert werden
	 * soll (-1 = keine Visualisierung).
	 * @return int visualize
	 */
	public int getVisualize()
	{
		return visualize;
	}

	/**
	 * Gibt den Teamnamen an.
	 * @return String
	 */
	public String getTeamName()
	{
		return teamName;
	}

	/**
	 * Gibt an, ob bei der Auswertung ein Syntaxfehler aufgetreten
	 * ist.
	 * @return boolean
	 */
	public boolean isSyntaxError()
	{
		return syntaxError;
	}
	/**
	 * Gibt die Anzahl der zu erstellenden Spieler an.
	 * @return int
	 */
	public int getPlayerNum()
	{
		return playerNum;
	}

	/**
	 * Gibt an, ob kein Torwart erzeugt werden soll.
	 * @return boolean
	 */
	public boolean isNoGoalie()
	{
		return noGoalie;
	}

	/**
	 * Gibt die Strategieklasse an.
	 * @return String
	 */
	public String getStrategyClass()
	{
		return strategyClass;
	}
}
