import java.util.*;

/**
 * <p>Diese Klasse zerlegt die Klammernnachrichten des RoboCup Servers in einzelne Teile. Dabei werden geschachtelte Klammern intakt gelassen.</p>
 *
 * <p>Beispiel:</p>
 * 
 * Sei die Nachricht "(see ((G) 0 300) ((B) 0 200) 20 40)"<br>
 * 
 * Dann entstehen folgende Teile:
 * <ul>
 * <li>"see"
 * <li>"((G) 0 300)"
 * <li>"((B) 0 200)"
 * <li>"20"
 * <li>"40"
 * </ul>
 * 
 * @author <a href="mailto:shantsch@rumms.uni-mannheim.de">Sebastian Hantsch</a>
 * @version 1.0
 */
public class ParenthesesTokenizer
{
	/** <code>String</code>, der die Originalnachricht enthlt. */
	private String msg;
	/** <code>Vector</code>, um die Teilstcke zu speichern. */
	private Vector tokens;
	/** <code>Enumeration</code>, um die Teilstcke auszulesen */
	private Enumeration enum;

	/**
	 * Erzeugt einen neuen <code>ParenthesesTokenizer</code>.
	 * 
	 * @param msg Die Nachricht, welche zerteilt werden soll
	 */
	public ParenthesesTokenizer(String msg)
	{
		tokens = new Vector(1);

		this.msg = msg;

		parseString(msg);

		enum = tokens.elements();
	}

	/**
	 * Nach Aufruf von <code>rewind</code> ist das erste Teilstck das nchste.
	 */
	public void rewind()
	{
		enum = tokens.elements();
	}

	/**
	 * Gibt das nchste Teilstck zurck.
	 */
	public String nextToken()
	{
		try
		{
			return (String) enum.nextElement();
		}
		catch (NoSuchElementException exc)
		{
			System.out.println("NoSuchElement: " + msg);
			exc.printStackTrace();
			return "";
		}
	}

	/**
	 * Gibt <code>true</code> zurck, falls weitere Teilstcke vorhanden sind,
	 * sonst <code>false</code>.
	 */
	public boolean hasMoreTokens()
	{
		return enum.hasMoreElements();
	}

	/**
	 * Gibt die Anzahl an Teilstcken zurck.
	 */
	public int tokenCount()
	{
		return tokens.size();
	}

	/**
	 * Gibt ein bestimmtes Teilstck zurck.
	 * @param tokenIndex Nummer des gewnschten Teilstcks
	 */
	public String getToken(int tokenIndex)
	{
		return (String) tokens.elementAt(tokenIndex);
	}

	/**
	 * Zerlegt eine Nachricht in Teilstcke.
	 */
	private void parseString(String str)
	{
		// aktuelle Klammerntiefe
		int parenthesesLevel = 0;

		StringBuffer curToken = new StringBuffer();

		for (int sPos = 0; sPos < str.length(); sPos++)
		{
			char curChar = str.charAt(sPos);
			boolean omitChar = false;

			switch (curChar)
			{
				case '(' :
					parenthesesLevel++;
					if (parenthesesLevel == 1)
						omitChar = true;
					break;
				case ')' :
					parenthesesLevel--;
					if (parenthesesLevel == 0)
						omitChar = true;
					break;
				case ' ' :
					if (parenthesesLevel == 1)
					{
						if (curToken.length() > 0)
						{
							tokens.add(new String(curToken));
							curToken.delete(0, curToken.length());
						}
						omitChar = true;
					}
					break;
			}

			if (!omitChar)
			{
				curToken.append(curChar);
			}
		}

		if (curToken.length() > 0)
		{
			tokens.add(new String(curToken));
		}
	}

	/**
	 * Gibt die originale Nachricht an, die zerlegt wurde.
	 * @return String msg
	 */
	public String getMsg()
	{
		return msg;
	}
}
