import javax.swing.*;
import java.awt.*;

/**
 * Spielfeldanzeige.
 * 
 * @author Jochen Rbenach
 * @version 1.21
 */

public class PlayFieldPanel extends JPanel
{
	private WorldModel worldModel;
	private int sizeX; //Hhe des Fensters
	private int sizeY; //Breite des Fenster

	final static double playerRadius = 1; //Gre eines Spielers

	/**
	 * Konstruktor
	 * 
	 * @param worldModel WorldModel
	 */
	public PlayFieldPanel(WorldModel worldModel)
	{
		super();
		this.worldModel = worldModel;

	}

	/**
	 * gibt von Panel Groesse zurueck
	 */
	public void calcSize()
	{
		sizeX = getBounds().width;
		sizeY = getBounds().height;
	}

	/**
	 * skaliert Punkte der X-Achse und verschiebt
	 * 
	 * @param x double
	 */
	private int scaleX(double x)
	{
		//wandle long wieder in int zurueck, round rundet Kommazahl
		return (int) Math.round((x + 60.0) * (sizeX / 120.0)); 
		//addiere auf gegebenen Wert 60 auf, fr die Hlfte des kompletten Feldes
		//und multipliziere dies mit dem Grenverhltnis bestehend aus der Fenster-
		//gre geteilt durch die Gesamtbreite des kompletten Feldes
	}

	/**
	 * skaliert Punkte der Y-Achse und verschiebt
	 * 
	 * @param y double
	 */
	private int scaleY(double y)
	{
		//wandle long wieder in int zurck, round rundet Kommazahl
		return (int) Math.round((y + 41.5) * (sizeY / 83.0));
		//quivalent zu scaleX
	}

	/**
	 * skaliert Strecken der X-Achse
	 * 
	 * @param x double
	 */
	private int scaleDistX(double x)
	{
		//wandle long wieder in int zurueck, round rundet Kommazahl
		return (int) Math.round(x * (sizeX / 120.0));
		//nehme gegebenen Wert und setze ihn ins Verhltnis zur Fenstergre 
	}

	/**
	 * skaliert Strecken der Y-Achse
	 * 
	 * @param y double
	 */
	private int scaleDistY(double y)
	{
		//wandle long wieder in int zurck, round rundet Kommazahl
		return (int) Math.round(y * (sizeY / 83.0));
		//nehme gegebenen Wert und setze ihn ins Verhltnis zur Fenstergre
	}

	/**
	 * malt das Spielfeld
	 * 
	 * @param g Graphics
	 */
	private void paintField(Graphics g)
	{
		double x = 0; //X-Position der Flagge
		double y = 0; //Y-Position der Flagge
		double d = 0.7; //Durchmesser der Flaggen
		double r = d / 2.0; //Radius der Flaggen

		//Spielfeld

		g.setColor(new Color(30, 170, 11)); //Farbe: grn
		g.fillRect(scaleX(-60), scaleY(-41.5), scaleDistX(120), scaleDistY(83)); //zeichnet grnes Spielfeld

		//Linien

		g.setColor(Color.white);
		//Physical boundary
		g.drawRect(scaleX(-57.5), scaleY(-39), scaleDistX(115), scaleDistY(78));
		//Spielfeldrand
		g.drawRect(scaleX(-52.5), scaleY(-34), scaleDistX(105), scaleDistY(68));
		//Mittellinie
		g.drawLine(scaleX(0), scaleY(-34), scaleX(0), scaleY(34));
		//Mittelkreis
		g.drawOval(scaleX(-9), scaleY(-9), scaleDistX(18), scaleDistY(18));
		//16-Meter-Raum links
		g.drawRect(
			scaleX(-52.5),
			scaleY(-20.175),
			scaleDistX(16.5),
			scaleDistY(40.35));
		//16-Meter-Raum rechts
		g.drawRect(
			scaleX(36),
			scaleY(-20.175),
			scaleDistX(16.5),
			scaleDistY(40.35));
		//5-Meter-Raum links	
		g.drawRect(
			scaleX(-52.5),
			scaleY(-9.3),
			scaleDistX(5.5),
			scaleDistY(18.6));
		//5-Meter-Raum rechts
		g.drawRect(scaleX(47), scaleY(-9.3), scaleDistX(5.5), scaleDistY(18.6));
		//Elfmeterpunkt links
		g.fillOval(
			scaleX(-41.5 - r),
			scaleY(0 - r),
			scaleDistX(d),
			scaleDistY(d));
		//Elfmeterpunkt
		g.fillOval(
			scaleX(41.5 - r),
			scaleY(0 - r),
			scaleDistX(d),
			scaleDistY(d));

		//Tore 

		//links
		g.fillRect(
			scaleX(-53.5),
			scaleY(-7.01),
			scaleDistX(1),
			scaleDistY(14.02));
		//rechts
		g.fillRect(
			scaleX(52.5),
			scaleY(-7.01),
			scaleDistX(1),
			scaleDistY(14.02));

		//Flaggen

		g.setColor(Color.black);

		for (x = -50; x <= 50; x += 10) //Physical boudary oben, unten
		{
			g.fillOval(
				scaleX(x - r),
				scaleY(-39 - r),
				scaleDistX(d),
				scaleDistY(d));
			g.fillOval(
				scaleX(x - r),
				scaleY(39 - r),
				scaleDistX(d),
				scaleDistY(d));
		}

		for (y = -30; y <= 30; y += 10) //Physical boundary links, rechts
		{
			g.fillOval(
				scaleX(-57.5),
				scaleY(y - r),
				scaleDistX(d),
				scaleDistY(d));
			g.fillOval(
				scaleX(57.5),
				scaleY(y - r),
				scaleDistX(d),
				scaleDistY(d));
		}

		for (y = -7.01; y <= 7.01; y += 7.01) //Tor links, rechts
		{
			g.fillOval(
				scaleX(-52.5),
				scaleY(y - r),
				scaleDistX(d),
				scaleDistY(d));
			g.fillOval(
				scaleX(52.5),
				scaleY(y - r),
				scaleDistX(d),
				scaleDistY(d));
		}

		for (y = -20.175;
			y <= 20.175;
			y += 20.175) //16-Meter-Raum links, rechts
		{
			g.fillOval(
				scaleX(-36),
				scaleY(y - r),
				scaleDistX(d),
				scaleDistY(d));
			g.fillOval(scaleX(36), scaleY(y - r), scaleDistX(d), scaleDistY(d));
		}

		for (x = -52.5; x <= 52.5; x += 52.5) //Spielfeldrand oben, unten
		{
			g.fillOval(
				scaleX(x - r),
				scaleY(-34 - r),
				scaleDistX(d),
				scaleDistY(d));
			g.fillOval(
				scaleX(x - r),
				scaleY(34 - r),
				scaleDistX(d),
				scaleDistY(d));
		}

		//Mittelpunkt
		g.fillOval(scaleX(0 - r), scaleY(0 - r), scaleDistX(d), scaleDistY(d));

	}

	/**
	 * zeichnet Spieler
	 * 
	 * @param g Graphics
	 */
	private void paintPlayers(Graphics g) 
	{
		//ausgewhlter Spieler
		Arrow pos = worldModel.getYourself().getPosition(); //Positon des Spielers/Ball
		Arrow vec = worldModel.getYourself().getVector(); //Vektor des Balls
		//Blickwinkel
		g.setColor(Color.red);
		final double viewRange = 200; //Lnge des eingezeichneten Blickwinkels, ist so lang, willkrlich gewhlt
		double xvektorUpperLine = 0; //x Vektor der oberen Begrenzungsgeraden
		double yvektorUpperLine = 0; //y Vektor der oberen Begrenzungsgeraden
		double xvektorDownLine = 0; //x Vektor der unteren Begrenzungsgeraden
		double yvektorDownLine = 0; //y Vektor der unteren Begrenzungsgeraden
		
		//Variablen zur Berechnung des Sichtwinkels
		double visibleAngel = 90; //setze visibleAngel am Anfang gleich 90
		String mode = worldModel.getYourself().getViewModeAngle(); //hole ViewModeAngel
		double viewangle = worldModel.getYourself().getViewAngle(); //Viewangle
		
		//prfe Strings, welcher Sichtwinkel gegeben ist und errechne & zeichne diesen
		if (mode.equals("narrow"))
		{
			visibleAngel = visibleAngel * 0.5;	//halbiere Sichtwinkel
			
			//bestimme obere Begrenzungsgerade des VisibleAngel=>subtrahiere halben visibleAngel
			xvektorUpperLine = Math.cos(Math.toRadians(viewangle - (visibleAngel / 2))) * viewRange;	
			yvektorUpperLine = Math.sin(Math.toRadians(viewangle - (visibleAngel / 2))) * viewRange;
			
			//bestimme untere Begrenzungsgerade des VisibleAngel=>addiere halben visibleAngel
			xvektorDownLine = Math.cos(Math.toRadians(viewangle + (visibleAngel / 2))) * viewRange;
			yvektorDownLine = Math.sin(Math.toRadians(viewangle + (visibleAngel / 2))) * viewRange;
			
			//zeichne 2 Gerade
			g.drawLine(
				scaleX(pos.getX()),
				scaleY(pos.getY()),
				scaleX(pos.getX() + xvektorUpperLine),
				scaleY(pos.getY() + yvektorUpperLine));
			g.drawLine(
				scaleX(pos.getX()),
				scaleY(pos.getY()),
				scaleX(pos.getX() + xvektorDownLine),
				scaleY(pos.getY() + yvektorDownLine));
		}
		else if (mode.equals("normal"))
		{
			visibleAngel = visibleAngel * 1.0;	// Sichtwinkel
			
			//bestimme obere Begrenzungsgerade des VisibleAngel=>subtrahiere halben visibleAngel
			xvektorUpperLine = Math.cos(Math.toRadians(viewangle - (visibleAngel / 2))) * viewRange;	
			yvektorUpperLine = Math.sin(Math.toRadians(viewangle - (visibleAngel / 2))) * viewRange;
			
			//bestimme untere Begrenzungsgerade des VisibleAngel=>addiere halben visibleAngel
			xvektorDownLine = Math.cos(Math.toRadians(viewangle + (visibleAngel / 2))) * viewRange;
			yvektorDownLine = Math.sin(Math.toRadians(viewangle + (visibleAngel / 2))) * viewRange;
			
			//zeichne 2 Gerade
			g.drawLine(
				scaleX(pos.getX()),
				scaleY(pos.getY()),
				scaleX(pos.getX() + xvektorUpperLine),
				scaleY(pos.getY() + yvektorUpperLine)); 
			g.drawLine(
				scaleX(pos.getX()),
				scaleY(pos.getY()),
				scaleX(pos.getX() + xvektorDownLine),
				scaleY(pos.getY() + yvektorDownLine));
		}
		else if (mode.equals("wide"))
		{
			visibleAngel = visibleAngel * 2.0;	//verdopple Sichtwinkel
		
			//bestimme obere Begrenzungsgerade des VisibleAngel=>subtrahiere halben visibleAngel
			xvektorUpperLine = Math.cos(Math.toRadians(viewangle - (visibleAngel / 2))) * viewRange;	
			yvektorUpperLine = Math.sin(Math.toRadians(viewangle - (visibleAngel / 2))) * viewRange;
		
			//bestimme untere Begrenzungsgerade des VisibleAngel=>addiere halben visibleAngel
			xvektorDownLine = Math.cos(Math.toRadians(viewangle + (visibleAngel / 2))) * viewRange;
			yvektorDownLine = Math.sin(Math.toRadians(viewangle + (visibleAngel / 2))) * viewRange;
		
			//zeichne 2 Gerade
			g.drawLine(
				scaleX(pos.getX()),
				scaleY(pos.getY()),
				scaleX(pos.getX() + xvektorUpperLine),
				scaleY(pos.getY() + yvektorUpperLine));
			g.drawLine(
				scaleX(pos.getX()),
				scaleY(pos.getY()),
				scaleX(pos.getX() + xvektorDownLine),
				scaleY(pos.getY() + yvektorDownLine));
		}
		
		//Zeichne Spieler
		g.setColor(new Color(253, 0, 203)); //Farbe: lila
		g.fillOval(
			scaleX(pos.getX() - playerRadius),
			scaleY(pos.getY() - playerRadius),
			scaleDistX(playerRadius * 2),
			scaleDistY(playerRadius * 2));
		
		// Ball zeichnen
		if (worldModel.isballSeen())
		{
			pos = worldModel.getBall().getPosition(); //hole Ballposition
			vec = worldModel.getBall().getVector(); //hole BallVektor
			g.setColor(Color.gray);
			//Zeichne Ball
			g.fillOval(
				scaleX(pos.getX() - playerRadius),
				scaleY(pos.getY() - playerRadius),
				scaleDistX(playerRadius * 2),
				scaleDistY(playerRadius * 2));
			g.setColor(Color.black);
			//Zeichne Richtungsvektor
			g.drawLine(
				scaleX(pos.getX()),
				scaleY(pos.getY()),
				scaleX(pos.getX() + vec.getX() * 4.0),
				scaleY(pos.getY() + vec.getY() * 4.0));
		}

		//eigene Spieler

		for (int i = 1; i <= worldModel.getPartyCount(); i++)
		{
			if (worldModel.getParty(i) != null)
			{

				pos = worldModel.getParty(i).getPosition(); //hole Position
				vec = worldModel.getParty(i).getVector(); //hole Vektor
				g.setColor(new Color(255, 2, 2)); //Farbe: rot
				g.fillOval(
					scaleX(pos.getX() - playerRadius),
					scaleY(pos.getY() - playerRadius),
					scaleDistX(playerRadius * 2),
					scaleDistY(playerRadius * 2));
			}
		}

		//Gegenspieler

		for (int i = 1; i <= worldModel.getEnemyCount(); i++)
		{
			if (worldModel.getEnemy(i) != null) //wenn existiert
			{

				pos = worldModel.getEnemy(i).getPosition(); //hole Position
				vec = worldModel.getEnemy(i).getVector(); //hole Vektor
				g.setColor(new Color(2, 14, 255)); //Farbe: blau
				g.fillOval(
					scaleX(pos.getX() - playerRadius),
					scaleY(pos.getY() - playerRadius),
					scaleDistX(playerRadius * 2),
					scaleDistY(playerRadius * 2));
			}
		}

		//unbekannte Spieler

		for (int i = 1; i <= worldModel.getUnknownsCount(); i++)
		{
			if (worldModel.getUnknowns(i) != null) //wenn existiert
			{
				pos = worldModel.getUnknowns(i).getPosition(); //hole Position
				vec = worldModel.getUnknowns(i).getVector(); //hole Vektor
				g.setColor(new Color(251, 243, 6)); //Farbe: gelb
				g.fillOval(
					scaleX(pos.getX() - playerRadius),
					scaleY(pos.getY() - playerRadius),
					scaleDistX(playerRadius * 2),
					scaleDistY(playerRadius * 2));
			}
		}
	}

	
	/**
	 * 
	 * @param g Graphics
	 */
	
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		if (worldModel != null) //wenn existiert
		{
			calcSize();
			paintField(g);
			paintPlayers(g);
		}
	}

	/**
	 * Sets the worldModel
	 * 
	 * @param worldModel WorldModel 
	 */
	public void setWorldModel(WorldModel worldModel)
	{
		this.worldModel = worldModel;
	}
}
