/**
 * Grundklasse mit allen allgemeinen Bewertungsfunktionen
 * 
 * @author Jrg Zentgraf
 * @version 1.0
 */

public abstract class Rating 
{
	/** Referenz auf WeltModell */
	protected WorldModel worldModel;
	/** Referenz auf Bewegungen */	
	protected Movement movement;

	/** aktuelles Kommando*/
	protected String command;
	/**Referenz auf eigenen Spieler */
	protected Hero hero;
	/** Referenz auf Ball */
	protected Things ball;
	/** Zyklenzhler*/
	protected int counter;

    /**
     * Konstruktor
     * @param worldModel Weltmodell
     * @param movement Movement
     * @param sender Sender   
     */
    public Rating(WorldModel worldModel, Movement movement, Sender sender) 
    {
   		this.worldModel=worldModel;
   		this.movement=movement;
    }
  
	/**
	 * Ist der Ball nah genug, dann kannst du ihn schiessen
	 * @return kickable
	 */   
	protected boolean kickable()
	{
		if (worldModel.getBall().getDistance()<0.8)
			return true;
		else
			return false;
	}	
	
	/**
	 * Ist der Ball in einer fr den Spieler relevanten Position ?
	 */
	protected boolean isRelevantPos()
	{
		int number = worldModel.getYourself().getNumber();
		Arrow ballPos = worldModel.getBall().getPosition();
		
		switch (number)
		{
			case 1:
				if (ballPos.getX()<0)
					return true;
				break;	
			case 2:
				if (ballPos.getX()<-20 && ballPos.getY()<-9)
					return true;
				break;	
			case 3:
				if (ballPos.getX()<-20 && Math.abs(ballPos.getY())<13)
					return true;
				break;			
			case 4:
				if (ballPos.getX()<-20 && ballPos.getY()>9)
					return true;
				break;	
			case 5:
				if (ballPos.getX()>-30 && ballPos.getX()<20
					&& ballPos.getY()<-15)
					return true;
				break;	
			case 6:
				if (ballPos.getX()>-30 && ballPos.getX()<20
					&& ballPos.getY()>-20 && ballPos.getY()<5)	
					return true;
				break;	
			case 7:
				if (ballPos.getX()>-30 && ballPos.getX()<20
					&& ballPos.getY()>-5 && ballPos.getY()<20)
					return true;
				break;	
			case 8:
				if (ballPos.getX()>-30 && ballPos.getX()<20
					&& ballPos.getY()>15)
					return true;
				break;	
			case 9:
				if (ballPos.getX()>-5 && ballPos.getY()<-9)
					return true;
				break;	
			case 10:
				if (ballPos.getX()>-5 && Math.abs(ballPos.getY())<9)
					return true;
				break;	
			case 11:
				if (ballPos.getX()>-5 && ballPos.getY()>9)
					return true;
				break;	
		}
		return false;
	}
	
	
	/**
	 * Prft, ob eine Spielunterbrechung vorliegt, die eine
	 * neue Aufstellung bentigt
	 */
	protected boolean isGameBreak()
	{
		String matchStatus = worldModel.getMatchStatus();

		return matchStatus.equals("before_kick_off")
			|| matchStatus.startsWith("goal_l")
			|| matchStatus.startsWith("goal_r");
	}	
	
	/**
	 * Liefert die Position eines Mitspielers, zu dem gepasst werden kann
	 * @return Arrow Position
	 */
	protected Arrow getKickTarget()
	{
		int plIndex = 0;
		double mindist = 100;
		Arrow myPos = worldModel.getYourself().getPosition();
		//default Position : gegnerisches Tor
		Arrow defPos = new Arrow(52,0);
		
		// Bei keinem Mitspieler return defPos
		if (worldModel.getPartyCount() < 1)
			return defPos;

		/* sonst checke bei jedem Mitspieler, ob dessen
			x-Wert grer ist als der eigene
			Bei allen Spielern, bei denen dies der Fall ist
			whle den Spieler mit der geringsten Entfernung
			*/ 
		else
		{
			for (int i = 1; i <= worldModel.getPartyCount(); i++)
			{
				Player pp = worldModel.getParty(i);
				if (pp != null)
				{
					//enemyCounter=10;
					if (pp.getPosition().getX() > myPos.getX()+2)
					{
						double dist = RoboCupMath.distance(myPos,pp.getPosition());
						if (dist < mindist)
						{
							mindist = dist;
							plIndex=i;
						}
					}
				}
			}

			if (plIndex > 0)
				return worldModel.getParty(plIndex).getPosition();
			else
				return defPos;
		}
	}

	   
	/**
	 * Sucht den Ball
	 */
	protected void searchBall()
   	{
		if (counter%2==0)
			movement.turn(60);
   	}
 	

	/**
	 * Berechnet die Position, an der wir den Ball abfangen knnen
	 * @return Arrow CollisionPosition
	 */
	protected Arrow getBallCollCourse()
	{
		Arrow myPos = worldModel.getYourself().getPosition();
		Arrow bPos = worldModel.getBall().getPosition();
		Arrow bVec = worldModel.getBall().getVector();
		Arrow futurePos = new Arrow(bPos.getX(), bPos.getY());
		
		double minFactor = 50;	
		double distPerCycle = 0.4;
		
		for (int i = 1; i <= 50; i++)
		{
			futurePos.addArrow(bVec);
			
			double curDist = RoboCupMath.distance(myPos, futurePos) - (double)i * distPerCycle;
			if (curDist < 0)
			{
				minFactor = i;
				break;
			}
		}
		return new Arrow(bPos.getX() + bVec.getX() * minFactor,
		                 bPos.getY() + bVec.getY() * minFactor);
	}
	
	/**
	 * Berechnet die Postion des Torwarts im Strafraum,
	 * an der er den Ball am besten abfangen kann
	 * @return Arrow CollisionPositionGoalie
	 */
	protected Arrow getBallCollCourseGoalie()
	{
		Arrow myPos = worldModel.getYourself().getPosition();
		Arrow bPos = worldModel.getBall().getPosition();
		Arrow bVec = worldModel.getBall().getVector();
		Arrow futurePos = new Arrow(bPos.getX(), bPos.getY());
		Arrow goalCatchPos = new Arrow(-48,0);
		
		double minFactor = 50;	
		double distPerCycle = 0.4;
		
		for (int i = 1; i <= 50; i++)
		{
			futurePos.addArrow(bVec);
			goalCatchPos.setY(futurePos.getY());
			
			double curDist = RoboCupMath.distance(goalCatchPos, futurePos);
			if (curDist < 0.8)
			{
				minFactor = i;
				break;
			}
		}
		if (goalCatchPos.getY()<-7)
			goalCatchPos.setY(-7);
		if 	(goalCatchPos.getY()>7)
			goalCatchPos.setY(7);
		return goalCatchPos;
	}	
	
	/**
	 * Berechnet ob der Ball im Strafraum ist
	 * @return penalty
	 */
	protected boolean penalty()
	{
		double ballx= ball.getPosition().getX();
		double bally= ball.getPosition().getY();
		if (ballx<-36 && Math.abs(bally)<20)
			return true;
		else
			return false;
	}	
	
 	/**
 	 * Berechnet, ob wir Abstoss haben
 	 * @return isOwnGoalKick
 	 */   
	protected boolean isOwnGoalKick()
	{
		String supposedMsg = "goal_kick_" + worldModel.getPlayDirection();
		return worldModel.getMatchStatus().equals(supposedMsg);
	}	 
	
	/**
	 * Berechnet, ob der Torwart den Ball gerade gefangen hat,
	 * d.h. es gibt Freistoss fr uns im Strafraum
	 * @return isOwnGoalieFreeKick
	 */
	protected boolean isOwnGoalieFreeKick()
	{
		String supposedMsg = "free_kick_" + worldModel.getPlayDirection();
		if (penalty())
			return worldModel.getMatchStatus().equals(supposedMsg);
		else	
			return false;	
	}	
	
	/**
	 * Wenn der Gegner eine Art von Freistoss hat, sollen sich unsere
	 * Spieler nicht bewegen
	 */
	protected void checkRun()
	{
		String matchStatus = worldModel.getMatchStatus();
		char enemySide = (worldModel.getPlayDirection() == 'l') ? 'r' : 'l';
		
		if (matchStatus.equals("kick_off_" + enemySide)
			|| matchStatus.equals("kick_in_" + enemySide)
			|| matchStatus.equals("goal_kick_" + enemySide)
			|| matchStatus.equals("corner_kick_" + enemySide)
			|| matchStatus.equals("free_kick_" + enemySide))		

			movement.setShouldRun(false);
		else
			movement.setShouldRun(true);
	} 
    
  
    /**
     * Wer hat Anspiel ?
     * 1 : Wir bereiten uns auf Anspiel vor
     * 2 : Wir drfen Anspielen
     * -1 : Gegner hat Anspiel
     * @return Anspiel
     */
    protected int kickoff()
    {
    	if (worldModel.getMatchStatus().equals("before_kick_off") &&
    		worldModel.getPlayDirection() == 'l')
     			return 1;
     	if (worldModel.getMatchStatus().equals("kick_off_l") &&
     		worldModel.getPlayDirection() == 'l')
     			return 2;		

     	if (worldModel.getMatchStatus().equals("kick_off_r") &&
     		worldModel.getPlayDirection() == 'r')
     			return 2;
     				
		if (worldModel.getMatchStatus().startsWith("goal_l") &&    		
			worldModel.getPlayDirection() == 'r')
				return 1;

		if (worldModel.getMatchStatus().startsWith("goal_r") &&
			worldModel.getPlayDirection() == 'l')
				return 1;	

    	return -1;
    }	
    
    /**
     * Hat jemand Anspiel ?
     * @return kickoff
     */
    
    protected boolean kickoff2()
    {
    	if (worldModel.getMatchStatus().startsWith("goal_l")
			|| worldModel.getMatchStatus().startsWith("goal_r"))
			return true;
		else
			return false;	
    }
    

	/**
	 * Whlt eine Strategie anhand der Ballposition aus
	 * und schreibt diese in command
	 */
    protected void chooseCommand()
    {
    	command="";
    	if (worldModel.isballSeen())
    	{
    		double ballx = worldModel.getBall().getPosition().getX();
    		double bally = worldModel.getBall().getPosition().getY();

    		if (ballx<-20)
    			command="ClearBall";
    		else
    		{
				if (ballx<25)
    				command="PassBall";
    				
	    		if (ballx>=25)
  	 	 			command="ScoreGoal";
    		}
    			

    	}
    	else
    		command = "SearchBall";
    }

	/**
	 * Vor jedem Anstoss dreht sich jeder Spieler in Richtung 0,0
	 */
	protected void startTurn()
	{
		if (counter%2==0)
			movement.turn(new Arrow(0,0));
	}
	
	/**Abstrakte Funktion fr die Ausfhrung der Bewertung jedes
	 * einzelnen Spielers */
	protected abstract void execute();
 	
}
