import javax.swing.*;

/**
 * <p>Diese Klasse erzeugt fr einen Spieler alle bentigten
 * Hauptklassen.</p>
 * 
 * 25.05.2002: Klasse umbenannt in RoboCup<br>
 * 26.05.2002: Aufrufsyntax aktualisiert<br>
 * 03.06.2002: Aufrufparameteranalyse ausgelagert in ParameterParser.java<br>
 * 06.06.2002: Neue Parameter bercksichtigt, Strategy wird dyn. erzeugt<br>
 * 08.06.2002: Neuer Aufrufsyntax von WorldModel bercksichtigt.<br>
 * 27.06.2002: Visualisierung wird nun vor den Spielern erzeugt.<br>
 * 
 * @author Sebastian Hantsch
 * @version 1.5
 */
public class RoboCup
{
	// Referenzen zu den bentigten Objekten
	CommunicationChannel commChannel;
	Receiver receiver;
	Sender sender;
	WorldModel worldModel;
	Strategy strategy;

	/**
	 * Erzeugt einen neuen Spieler.
	 * @param ParameterParser paraParser Kommandozeilenoptionen
	 * @param boolean isGoalie Spieler als Torwart erzeugen
	 */
	public RoboCup(ParameterParser paraParser, boolean isGoalie)
	{
		boolean strategyFound;
		// Weltmodell erzeugen
		worldModel = new WorldModel(paraParser.getTeamName(), isGoalie);
		
		// Netzwerkkomponente erzeugen
		commChannel =
			new CommunicationChannel(
				paraParser.getServerName(),
				paraParser.getServerPort());

		// Sender erzeugen
		sender = new Sender(commChannel);

		// Strategie erzeugen
		strategyFound = true;
		try
		{
			strategy =
				(Strategy) Class
					.forName(paraParser.getStrategyClass())
					.newInstance();
		}
		catch (Exception e)
		{
			strategyFound = false;
		}
		if (strategyFound)
		{
			// Referenzen der Strategie setzen
			strategy.setReferences(worldModel, sender);
			
			// Empfnger erzeugen...
			receiver = new Receiver(commChannel, worldModel, strategy);

			// ...und den Empfnger starten
			receiver.start();
			
			// beim Server anmelden
			sender.init(paraParser.getTeamName(), isGoalie);
		}
		else
		{
			System.err.println(
				"Fehler beim Erzeugen der Strategie \""
					+ paraParser.getStrategyClass()
					+ "\"");
		}
	}

	/**
	 * Gibt den <code>Receiver</code> des Spielers an.
	 * @return Receiver
	 */
	public Receiver getReceiver()
	{
		return receiver;
	}

	/**
	 * Gibt das <code>WorldModel</code> des Spielers an.
	 * @return WorldModel
	 */
	public WorldModel getWorldModel()
	{
		return worldModel;
	}

	/**
	 * Hier werden Spieler erzeugt (abhngig von der Kommandozeile).<br>
	 * Aufrufsyntax des Programms siehe @see ParameterParser
	 */
	public static void main(String[] args)
	{
		/** Kommandozeilenparameter auswerten */
		ParameterParser paraParser = new ParameterParser(args);
		RoboCup[] roboCup;
		Visualization visualization;

		if (paraParser.isSyntaxError())
		{
			// Syntaxfehler, also Fehlermeldung ausgeben
			ParameterParser.printSyntax(System.err);
		}
		else
		{			
			// Spielervektor erzeugen
			roboCup = new RoboCup[paraParser.getPlayerNum()];

			// ggf. Visualization erzeugen
			if (paraParser.getVisualize() != -1)
			{
				// Look&Feel von Swing setzen
				try
				{
					UIManager.setLookAndFeel(
						UIManager.getCrossPlatformLookAndFeelClassName());
				}
				catch (Exception e)
				{
				}
				
				// Visualisierung erzeugen
				visualization =
					new Visualization(roboCup, paraParser.getVisualize() - 1);
			}
			else
			{
				visualization = null;
			}
			
			// Spieler erzeugen				
			for (int i = 0; i < paraParser.getPlayerNum(); i++)
			{
				roboCup[i] =
					new RoboCup(
						paraParser,
						(i == 0 && !paraParser.isNoGoalie()));
			}
			
			// Visualisierung auf gewhlten Spieler einstellen
			if (visualization != null)
			{
				visualization.visualizePlayer(paraParser.getVisualize() - 1);
			}
		}
	}
}