/**
 * Diese Klasse enthlt (statische) Methoden zur Berechnung
 * von Abstnden, Winkeln usw.
 * 
 * @author Sebastian Hantsch
 * @version 1.1
 */
public final class RoboCupMath
{

	/**
	 * Von dieser Klasse sollen keine Instanzen erstellt werden.
	 */
	private RoboCupMath()
	{
	}

	/**
	 * Berechnet die Entfernung von zwei Punkten a und b
	 * @param Arrow a
	 * @param Arrow b
	 */
	public static double distance(Arrow a, Arrow b)
	{
		double dx = a.getX() - b.getX();
		double dy = a.getY() - b.getY();
		
		return Math.sqrt(dx * dx + dy * dy);
	}
	
	/**
	 * Berechnet den Winkel zwischen zwei Punkten a und b
	 * @param Arrow a
	 * @param Arrow b
	 */
   	public static double angle(Arrow a, Arrow b)
	{
		double dx = a.getX() - b.getX();
		double dy = a.getY() - b.getY();
		double ang = Math.toDegrees(Math.atan(dy / dx));
		Double test = new Double(ang);
		if (test.isNaN())
		{
			ang=0;
		}
		
		
		if (dx < 0)
		{
			ang += 180;
		}
		
		if (ang > 180)
		{
			ang -= 360;
		}
		
		return ang;
	}
	
	/**
	 * Berechnet die Differenz zwischen zwei Winkeln
	 * @param double w1
	 * @param double w2
	 */
	public static double angleDiff(double w1, double w2)
	{
		
		// Winkel auf 0-360 translieren
		
		if (w1 < 0)
			w1 += 360.0;
		if (w2 < 0)
			w2 += 360.0;
			
		double diff = w2 - w1;
		if (diff > 180)
		{
			diff -= 360;
		}
		else if (diff < -180)
		{
			diff += 360;
		}
		Double test = new Double(diff);
		if (test.isNaN())
		{
			diff=0;
		}	
		
		return diff;		
	}	
}
