/**
 * Diese Klasse implementiert das Laufen eines Spielers.
 * 
 * @author Sebastian Hantsch
 * @version 1.1
 */
public class RunAction extends BasicAction
{
	/** Ziel des Spielers */
	private Arrow destination;
	/** Zeitpunkt des letzten turn Befehls */
	private int lastTurn;

	/**
	 * Erzeugt eine neue <code>RunAction</code>.
	 * @param worldModel Referenz zum Weltmodell.
	 * @param sender Referenz zum Sender.
	 */
	public RunAction(WorldModel worldModel, Sender sender)
	{
		super(worldModel, sender);
		destination = new Arrow();
		lastTurn = 0;
	}

	/**
	 * Setzt das Ziel fr den Spieler
	 * @param destination das Ziel
	 */
	public void setDestination(Arrow destination)
	{
		this.destination.setXY(destination.getX(), destination.getY());
	}

	/**
	 * Aktion fortsetzen.
	 */
	public void resume()
	{
		// Prfen, ob richtiger Winkel zum Ziel
		Hero me = worldModel.getYourself();
		double istWinkel = me.getViewAngle() - me.getHeadAngle();
		double sollWinkel = RoboCupMath.angle(destination, me.getPosition());
		double winkelAbw;
		double drehwinkel;
		double dist = RoboCupMath.distance(me.getPosition(), destination);
			
		// Nahe genug am Ziel?
		if (dist < 0.5)
		{
			// Ja, also Aktion beenden
			actionFinished = true;
			actionPending = false;
		}
		else		
		{
			// Nein, also muss noch gelaufen werden
			
			// ggf. den Kopf nach vorne drehen
			double headAngle = worldModel.getYourself().getHeadAngle();
			if (headAngle != 0)
			{
				sender.turnNeck(-headAngle);
			}
			
			// Spieler in Richtung Ziel drehen
			winkelAbw = RoboCupMath.angleDiff(istWinkel, sollWinkel);

			if (Math.abs(winkelAbw) > 5)
			{
				drehwinkel = winkelAbw;

				// Allerdings nicht zuviel auf einmal drehen
				if (drehwinkel < 0)
				{
					drehwinkel = Math.max(drehwinkel, -160);
				}
				else
				{
					drehwinkel = Math.min(drehwinkel, 160);
				}

				// falls letzte Drehung ausreichend zurckliegt,
				// neue Drehung durchfhren
				if (lastTurn < worldModel.getYourself().getTime() - 2)
				{
					sender.turn(drehwinkel);
					lastTurn = worldModel.getYourself().getTime();
				}
				else
				{
					if (Math.abs(drehwinkel) < 45)
					{
						sender.dash(50);
					}
				}
			}
			else
			{
				// Winkel richtig, also loslaufen
				if (dist > 0.8)
				{
					sender.dash(100);
				}
				else
				{
					sender.dash(70);
				}
			}
		}
	}
}
