import java.util.*;

/**
 * Umsetzung der see-Nachricht als Klasse.
 * 
 * 03.6.2002: processTokens eingefgt
 * 05.6.2002: goals und flags zusammengefasst
 * 
 * @author Sebastian Hantsch
 * @version 1.2
 */
public class SeeEvent extends TimeStampedEvent
{
	private Vector lines;
	private Vector flags;
	private Vector balls;
	private Vector players;

	/**
	 * Erzeugt ein neues SeeEvent.
	 */
	public SeeEvent()
	{
		super();
		lines = new Vector();
		flags = new Vector();
		balls = new Vector();
		players = new Vector();
	}

	/**
	 * Liest die Teilstcke von tokens aus und belegt die
	 * Ereignisattribute entsprechend
	 * @param tokens Teilstcke
	 * @return <code>true</code>, falls gltige Anzahl/Art von 
	 * Token, sonst <code>false</code>
	 */
	public boolean processTokens(ParenthesesTokenizer tokens)
	{
		if (super.processTokens(tokens))
		{
			lines.clear();
			flags.clear();
			balls.clear();
			players.clear();

			while (tokens.hasMoreTokens())
			{
				String seenObject = tokens.nextToken();
				SeeObject seeObject = new SeeObject(seenObject);

				// seeObject einer Typliste zuordnen
				if (seeObject.isLine())
				{
					lines.addElement(seeObject);
				}
				else if (seeObject.isFlag() || seeObject.isGoal())
				{
					flags.addElement(seeObject);
				}
				else if (seeObject.isBall())
				{
					balls.addElement(seeObject);
				}
				else if (seeObject.isPlayer())
				{
					players.addElement(seeObject);
				}
				else
				{
					return false;
				}
			}
			return true;
		}
		return false;
	}

	/**
	 * Gibt die Anzahl an Linien des <code>SeeEvent</code> an.
	 */
	public int lineCount()
	{
		return lines.size();
	}

	/**
	 * Gibt eine bestimmte Linie als <code>SeeObject</code> an.
	 * @param index Index der gewnschten Linie
	 */
	public SeeObject getLine(int index)
	{
		return (SeeObject) lines.elementAt(index);
	}

	/**
	 * Gibt die Anzahl an Flaggen des <code>SeeEvent</code> an.
	 */
	public int flagCount()
	{
		return flags.size();
	}

	/**
	 * Gibt eine bestimmte Flagge als <code>SeeObject</code> an.
	 * @param index Index der gewnschten Flagge
	 */
	public SeeObject getFlag(int index)
	{
		return (SeeObject) flags.elementAt(index);
	}

	/**
	 * Gibt die Anzahl an Bllen des <code>SeeEvent</code> an.
	 */
	public int ballCount()
	{
		return balls.size();
	}

	/**
	 * Gibt einen bestimmten Ball als <code>SeeObject</code> an.
	 * @param index Index des gewnschten Balls
	 */
	public SeeObject getBall(int index)
	{
		return (SeeObject) balls.elementAt(index);
	}

	/**
	 * Gibt die Anzahl an Spielern des <code>SeeEvent</code> an.
	 */
	public int playerCount()
	{
		return players.size();
	}

	/**
	 * Gibt ein bestimmten Spieler als <code>SeeObject</code> an.
	 * @param index Index des gewnschten Spielers
	 */
	public SeeObject getPlayer(int index)
	{
		return (SeeObject) players.elementAt(index);
	}
}