import java.util.*;

/**
 * Diese Klasse beschreibt ein Objekt, welches durch eine see Nachricht
 * empfangen wurde.
 * 
 * 08.06.2002: Bei p-Objekten werden nun Teamname, Spielernummer und Torwart
 *             ausgewertet <br>
 * 
 * @author Sebastian Hantsch
 * @version 1.1
 */
public class SeeObject
{
	// Attribute des gesehenen Objekts, vergleiche das Server-Manual
	private String objName;		
	private double direction;
	private boolean directionAvailable;
	private double distance;
	private boolean distanceAvailable;	
	private double distChange;
	private boolean distChangeAvailable;	
	private double dirChange;
	private boolean dirChangeAvailable;	
	private double bodyFacingDir;
	private boolean bodyFacingDirAvailable;	
	private double headFacingDir;
	private boolean headFacingDirAvailable;	
	
	// Attribute bei p
	private String teamName;
	private boolean teamNameAvailable;
	private int uniformNumber;
	private boolean uniformNumberAvailable;
	private boolean isGoalie;
	private boolean isGoalieAvailable;

	public SeeObject( String seenObject )
	{
		// *Available auf false setzen
		directionAvailable = false;
		distanceAvailable = false;
		distChangeAvailable = false;
		dirChangeAvailable = false;
		bodyFacingDirAvailable = false;
		headFacingDirAvailable = false;
		teamNameAvailable = false;
		uniformNumberAvailable = false;
		isGoalieAvailable = false;
		teamName = "";

		ParenthesesTokenizer seenTokens = new ParenthesesTokenizer(seenObject);

		objName = seenTokens.nextToken();
		// Klammern entfernen
		objName = objName.substring(1, objName.length() - 1);

		if (seenTokens.hasMoreTokens())
		{
			direction = Double.parseDouble(seenTokens.nextToken());
			directionAvailable = true;
		}

		if (seenTokens.hasMoreTokens())
		{
			distance = direction;
			direction = Double.parseDouble(seenTokens.nextToken());
			distanceAvailable = true;
		}

		if (seenTokens.hasMoreTokens())
		{
			distChange = Double.parseDouble(seenTokens.nextToken());
			distChangeAvailable = true;			
		}
		
		if (seenTokens.hasMoreTokens())
		{
			dirChange = Double.parseDouble(seenTokens.nextToken());
			dirChangeAvailable = true;			
		}
		
		if (seenTokens.hasMoreTokens())
		{
			bodyFacingDir = Double.parseDouble(seenTokens.nextToken());
			bodyFacingDirAvailable = true;			
		}
		
		if (seenTokens.hasMoreTokens())
		{
			headFacingDir = Double.parseDouble(seenTokens.nextToken());			
			headFacingDirAvailable = true;			
		}
		
		// p verarbeiten
		if (isPlayer())
		{
			StringTokenizer st = new StringTokenizer(objName);
			if (st.hasMoreTokens())
			{
				String pStr = st.nextToken();
				
				if (pStr.equals("p"))
				{
					if (st.hasMoreTokens())
					{
						teamName = st.nextToken();
						// Anfhrungszeichen entfernen
						teamName = teamName.substring(1, teamName.length() - 1);
						teamNameAvailable = true;
					}
					
					if (st.hasMoreTokens())
					{
						uniformNumber = Integer.parseInt(st.nextToken());
						uniformNumberAvailable = true;						
					}	
					
					if (st.hasMoreTokens())
					{
						String gStr = st.nextToken();
						isGoalie = gStr.equals("goalie");
						isGoalieAvailable = true;
					}
				}
			}
		}
	}
	
	public String getObjName()
	{
		return objName;
	}
	
	public double getDirection()
	{
		return direction;
	}
	
	public double getDistance()
	{
		return distance;
	}
	
	public double getDistChange()
	{
		return distChange;
	}
	
	public double getDirChange()
	{
		return dirChange;
	}
	
	public double getBodyFacingDir()
	{
		return bodyFacingDir;
	}
	
	public double getHeadFacingDir()
	{
		return headFacingDir;
	}
	
	public boolean isLine()
	{
		return objName.startsWith( "l" );
	}

	public boolean isFlag()
	{
		return objName.startsWith( "f" ) || objName.startsWith( "F" );
	}

	public boolean isBall()
	{
		return objName.startsWith( "b" ) || objName.startsWith( "B" );
	}

	public boolean isGoal()
	{
		return objName.startsWith( "g" ) || objName.startsWith( "G" );		
	}

	public boolean isPlayer()
	{
		return objName.startsWith( "p" ) || objName.startsWith( "P" );
	}

	/**
	 * Gets the directionAvailable.
	 * @return Returns a boolean
	 */
	public boolean getDirectionAvailable()
	{
		return directionAvailable;
	}

	/**
	 * Gets the distanceAvailable.
	 * @return Returns a boolean
	 */
	public boolean getDistanceAvailable()
	{
		return distanceAvailable;
	}

	/**
	 * Gets the distChangeAvailable.
	 * @return Returns a boolean
	 */
	public boolean getDistChangeAvailable()
	{
		return distChangeAvailable;
	}

	/**
	 * Gets the dirChangeAvailable.
	 * @return Returns a boolean
	 */
	public boolean getDirChangeAvailable()
	{
		return dirChangeAvailable;
	}

	/**
	 * Gets the bodyFacingDirAvailable.
	 * @return Returns a boolean
	 */
	public boolean getBodyFacingDirAvailable()
	{
		return bodyFacingDirAvailable;
	}

	/**
	 * Gets the headFacingDirAvailable.
	 * @return Returns a boolean
	 */
	public boolean getHeadFacingDirAvailable()
	{
		return headFacingDirAvailable;
	}

	/**
	 * Returns the isGoalie.
	 * @return boolean
	 */
	public boolean isGoalie()
	{
		return isGoalie;
	}

	/**
	 * Returns the isGoalieAvailable.
	 * @return boolean
	 */
	public boolean isGoalieAvailable()
	{
		return isGoalieAvailable;
	}

	/**
	 * Returns the teamName.
	 * @return String
	 */
	public String getTeamName()
	{
		return teamName;
	}

	/**
	 * Returns the teamNameAvailable.
	 * @return boolean
	 */
	public boolean isTeamNameAvailable()
	{
		return teamNameAvailable;
	}

	/**
	 * Returns the uniformNumber.
	 * @return int
	 */
	public int getUniformNumber()
	{
		return uniformNumber;
	}

	/**
	 * Returns the uniformNumberAvailable.
	 * @return boolean
	 */
	public boolean isUniformNumberAvailable()
	{
		return uniformNumberAvailable;
	}
}
