import java.io.*;

/**
 * Diese Klasse dient zum Generieren von Clientnachrichten.
 * Sie bildet alle Clientkommandos ab, die im Servermanual
 * beschrieben werden.
 * 
 * @author Sebastian Hantsch
 * @version 1.0
 */
public class Sender
{
	/** Referenz zur Netzwerkkomponente */
	CommunicationChannel commChannel;

	/**
	 * Erzeugt einen neuen Sender.
	 * @param CommunicationChannel commChannel Referenz zur Netzwerkkomponente
	 */	
	public Sender(CommunicationChannel commChannel)
	{
		this.commChannel = commChannel;
	}

	/**
	 * Sende init Kommando.
	 * @param String teamName Teamname des Spielers
	 * @param boolean isGoalie Spieler soll Torwart sein
	 */
	public void init(String teamName, boolean isGoalie)
	{
		if( isGoalie )
		{
			commChannel.send( new String( "(init " + teamName + "(version 7.0) (goalie))" ) );
		}
		else
		{
			commChannel.send( new String( "(init " + teamName + "(version 7.0))" ) );
		}
	}
	
	/**
	 * Sende reconnect Kommando.
	 * @param String teamName Teamname des Spielers
	 * @param int unum Spielernummer
	 */
	public void reconnect(String teamName, int unum)
	{
		commChannel.send( new String( "(reconnect " + teamName + " " + unum + ")" ) );
	}
	
	/**
	 * Sende bye Kommando.
	 */
	public void bye()
	{
		commChannel.send("(bye)");
	}
	
	/**
	 * Sende catch Kommando.
	 * @param double direction Fangrichtung
	 */
	public void doCatch(double direction)
	{
		commChannel.send( new String( "(catch " + direction + ")" ) );
	}
	
	/**
	 * Sende changeview Kommando.
	 * @param String width Grsse des Sichtfeldes
	 * @param String quality Sichtqualitt
	 */
	public void changeView(String width, String quality)
	{
		commChannel.send( new String( "(change_view " + width + " " + quality + ")" ) );
	}

	/**
	 * Sende dash Kommando.
	 * @param double power Beschleunigungsstrke
	 */
	public void dash(double power)
	{
		commChannel.send( new String( "(dash " + power + ")" ) );
	}

	/**
	 * Sende kick Kommando.
	 * @param double power Schussstrke
	 * @param double direction Schussrichtung
	 */
	public void kick(double power, double direction)
	{
		commChannel.send( new String( "(kick " + power + " " + direction + ")" ) );
	}

	/**
	 * Sende move Kommando.
	 * @param double x Neue x-Koordinate des Spielers
	 * @param double y Neue y-Koordinate des Spielers
	 */
	public void move(double x, double y)
	{
		commChannel.send( new String( "(move " + x + " " + y + ")" ) );
	}
	
	/**
	 * Sende say Kommando.
	 * @param String msg Zu sendende Nachricht
	 */
	public void say(String msg)
	{
		commChannel.send( new String( "(say " + msg + ")" ) );
	}
	
	/**
	 * Sende sense_body Kommando.
	 */
	public void senseBody()
	{
		commChannel.send( "(sense_body)" );
	}
	
	/**
	 * Sende score Kommando.
	 */
	public void score()
	{
		commChannel.send( "(score)" );
	}		

	/**
	 * Sende turn Kommando.
	 * @param double moment Drehmoment
	 */
	public void turn(double moment)
	{
		commChannel.send( new String( "(turn " + moment + ")" ) );
	}

	/**
	 * Sende turn_neck Kommando.
	 * @param double angle Kopfdrehwinkel
	 */
	public void turnNeck(double angle)
	{
		commChannel.send( new String( "(turn_neck " + angle + ")" ) );
	}
}