/**
 * Umsetzung der sense_body-Nachricht als Klasse.
 * 
 * 03.6.2002: processTokens eingefgt
 *  
 * @author Sebastian Hantsch
 * @version 1.1
 */
public class SenseBodyEvent extends TimeStampedEvent
{
	// Attribute des sense_body - Ereignisses
	// (vgl. das Servermanual)
	private String viewModeInterval;
	private String viewModeAngle;
	private double stamina;
	private double effort;
	private double amountOfSpeed;
	private double directionOfSpeed;
	private double headAngle;
	private int kickCount;
	private int dashCount;
	private int turnCount;
	private int sayCount;
	private int turnNeckCount;
	private int catchCount;
	private int moveCount;
	private int changeViewCount;

	/**
	 * Erzeugt ein neues SenseBodyEvent.
	 */
	public SenseBodyEvent()
	{
		super();
		viewModeInterval = "";
		viewModeAngle = "";
	}

	/**
	 * Liest die Teilstcke von tokens aus und belegt die
	 * Ereignisattribute entsprechend
	 * @param tokens Teilstcke
	 * @return <code>true</code>, falls gltige Anzahl/Art von 
	 * Token, sonst <code>false</code>
	 */
	public boolean processTokens(ParenthesesTokenizer tokens)
	{
		if (super.processTokens(tokens))
		{

			ParenthesesTokenizer subTokens;

			try
			{

				// view_mode
				subTokens = new ParenthesesTokenizer(tokens.nextToken());
				viewModeInterval = subTokens.getToken(1);
				viewModeAngle = subTokens.getToken(2);

				// stamina
				subTokens = new ParenthesesTokenizer(tokens.nextToken());
				stamina = Double.parseDouble(subTokens.getToken(1));
				effort = Double.parseDouble(subTokens.getToken(2));

				// speed
				subTokens = new ParenthesesTokenizer(tokens.nextToken());
				amountOfSpeed = Double.parseDouble(subTokens.getToken(1));
				directionOfSpeed = Double.parseDouble(subTokens.getToken(2));

				// head_angle		
				subTokens = new ParenthesesTokenizer(tokens.nextToken());
				headAngle = Double.parseDouble(subTokens.getToken(1));

				// kick
				subTokens = new ParenthesesTokenizer(tokens.nextToken());
				kickCount = Integer.parseInt(subTokens.getToken(1));

				// dash
				subTokens = new ParenthesesTokenizer(tokens.nextToken());
				dashCount = Integer.parseInt(subTokens.getToken(1));

				// turn		
				subTokens = new ParenthesesTokenizer(tokens.nextToken());
				turnCount = Integer.parseInt(subTokens.getToken(1));

				// say
				subTokens = new ParenthesesTokenizer(tokens.nextToken());
				sayCount = Integer.parseInt(subTokens.getToken(1));

				// turn_neck
				subTokens = new ParenthesesTokenizer(tokens.nextToken());
				turnNeckCount = Integer.parseInt(subTokens.getToken(1));

				// catch
				subTokens = new ParenthesesTokenizer(tokens.nextToken());
				catchCount = Integer.parseInt(subTokens.getToken(1));

				// move
				subTokens = new ParenthesesTokenizer(tokens.nextToken());
				moveCount = Integer.parseInt(subTokens.getToken(1));

				// change_view
				subTokens = new ParenthesesTokenizer(tokens.nextToken());
				changeViewCount = Integer.parseInt(subTokens.getToken(1));
				return true;
			}
			catch (Exception e)
			{
				e.printStackTrace();
				return false;
			}
		}
		return false;
	}

	/**
	 * Gibt die Hufigkeit des Sehens an.
	 * @return viewModeInterval
	 */
	public String getViewModeInterval()
	{
		return viewModeInterval;
	}

	/**
	 * Gibt den Sichtwinkelmodus an.
	 * @return viewModeAngle
	 */
	public String getViewModeAngle()
	{
		return viewModeAngle;
	}

	/**
	 * Gibt die Ausdauer an.
	 * @return stamina
	 */
	public double getStamina()
	{
		return stamina;
	}

	/**
	 * Gibt die Anstrengung an.
	 * @return effort
	 */
	public double getEffort()
	{
		return effort;
	}

	/**
	 * Gibt die Geschwindigkeit an.
	 * @return amountOfSpeed
	 */
	public double getAmountOfSpeed()
	{
		return amountOfSpeed;
	}

	/**
	 * Gibt die Richtung an.
	 * @return directionOfSpeed
	 */
	public double getDirectionOfSpeed()
	{
		return directionOfSpeed;
	}

	/**
	 * Gibt den Kopfwinkel an.
	 * @return headAngle
	 */
	public double getHeadAngle()
	{
		return headAngle;
	}

	/**
	 * Gibt die Anzahl an kick-Befehlen an.
	 * @return kickCount
	 */
	public int getKickCount()
	{
		return kickCount;
	}

	/**
	 * Gibt die Anzahl an dash-Befehlen an.
	 * @return dashCount
	 */
	public int getDashCount()
	{
		return dashCount;
	}

	/**
	 * Gibt die Anzahl an turn-Befehlen an.
	 * @return turnCount
	 */
	public int getTurnCount()
	{
		return turnCount;
	}

	/**
	 * Gibt die Anzahl an say-Befehlen an.
	 * @return sayCount
	 */
	public int getSayCount()
	{
		return sayCount;
	}

	/**
	 * Gibt die Anzahl an turn_neck-Befehlen an.
	 * @return turnNeckCount
	 */
	public int getTurnNeckCount()
	{
		return turnNeckCount;
	}

	/**
	 * Gibt die Anzahl an catch-Befehlen an.
	 * @return catchCount
	 */
	public int getCatchCount()
	{
		return catchCount;
	}

	/**
	 * Gibt die Anzahl an move-Befehlen an.
	 * @return moveCount
	 */
	public int getMoveCount()
	{
		return moveCount;
	}

	/**
	 * Gibt die Anzahl an change_view-Befehlen an.
	 * @return changeViewCount
	 */
	public int getChangeViewCount()
	{
		return changeViewCount;
	}
}