/**
 * Diese Klasse dient zum Verschicken von Spielernachrichten
 * ber das say Kommando.
 * 
 * @author Sebastian Hantsch
 * @version 1.0
 */
public class Speaker
{
	/** Referenz zum Sender */
	private Sender sender;
	/** Referenz zum Weltmodell */
	private WorldModel worldModel;
	/** Anzahl der bereits gesendeten Nachrichten */
	private int msgCount;
	/** Zeitpunkt des letzten Sendevorgangs */
	private int lastMsgTime;
	/** Hashcode des eigenen Teamnamens */
	private int teamHash;

	/**
	 * Erzeugt einen neuen <code>Speaker</code>.
	 * @param WorldModel worldModel Weltmodell-Referenz
	 * @param Sender sender Sender-Referenz
	 */
	public Speaker(WorldModel worldModel, Sender sender)
	{
		// Referenzen setzen
		this.worldModel = worldModel;
		this.sender = sender;
		
		// Hashcode des Teamnamens bestimmen
		teamHash = worldModel.getTeamName().hashCode();

		// Nachrichtenzahl und Zeitpunkt des letzten
		// Sendevorgangs initialisieren.
		msgCount = 0;
		lastMsgTime = 0;
	}

	/**
	 * Erneuert den Inhalt von msgCount und lastMsgTime.
	 */
	private void updateState()
	{
		msgCount++;
		lastMsgTime = worldModel.getActTime();
	}

	/**
	 * Erzeugt den allgemeinen Nachrichtenkopf. Dieser
	 * besteht aus dem Hashcode des Teamnamens, der eigenen
	 * Spielernummer, der Nachrichtenzahl sowie der Sendezeit.
	 */
	private String msgHead()
	{
		return ""
			+ teamHash
			+ " "
			+ worldModel.getYourself().getNumber()
			+ " "
			+ msgCount
			+ " "
			+ worldModel.getActTime();
	}

	/**
	 * Sendet eine Ballpositionsnachricht.
	 */
	public void reportBallPos()
	{
		Arrow bPos = worldModel.getBall().getPosition();
		Arrow bVec = worldModel.getBall().getVector();

		sender.say(
			"("
				+ msgHead()
				+ " B "
				+ bPos.getX()
				+ " "
				+ bPos.getY()
				+ " "
				+ bVec.getX()
				+ " "
				+ bVec.getY()
				+ ")");
		updateState();
	}

	/**
	 * Sendet eine Ballbesitznachricht.
	 */
	public void reportBallOwnership()
	{
		sender.say("(" + msgHead() + " BO)");
		updateState();
	}

	/**
	 * Gibt den Zeitpunkt des letzten Nachrichtenversands an.
	 * @return int lastMsgTime
	 */
	public int getLastMsgTime()
	{
		return lastMsgTime;
	}
}
