/**
 * Abstrakte Klasse fr alle Server-Ereignisse, die einen Timestamp enthalten.
 * 
 * 27.5.2002: Test auf ausreichend Tokens eingebaut.
 * 03.6.2002: processTokens eingefgt
 * 
 * @author Sebastian Hantsch
 * @version 1.2
 */
public abstract class TimeStampedEvent extends ServerEvent
{
	/** <code>int</code>, der den Timestamp enthlt. */
	protected int timeStamp;

	/**
	 * Erzeugt ein neues TimeStampedEvent.
	 */
	public TimeStampedEvent()
	{
		super();
	}

	/**
	 * Liest die Teilstcke von tokens aus und belegt die
	 * Ereignisattribute entsprechend
	 * @param tokens Teilstcke
	 * @return <code>true</code>, falls gltige Anzahl/Art von 
	 * Token, sonst <code>false</code>
	 */
	public boolean processTokens(ParenthesesTokenizer tokens)
	{
		if (super.processTokens(tokens))
		{
			if (tokens.hasMoreTokens())
			{
				timeStamp = Integer.parseInt(tokens.nextToken());
				return true;
			}
		}
		return false;
	}

	/**
	 * Gibt den Timestamp an.
	 * @return Timestamp
	 */
	public int getTimeStamp()
	{
		return timeStamp;
	}
}