/**
 * Diese Klasse implementiert das Drehen eines Spielers.
 * 
 * @author Sebastian Hantsch
 * @version 1.0
 */
public class TurnAction extends BasicAction
{
	/** Zeitpunkt des letzten turn Befehls */
	private int lastTurn = 0;
	/** Zielwinkel */
	private double destWinkel;
	/** Kopfwinkel zurcksetzen */
	private boolean resetHeadAngle;

	/**
	 * Erzeugt eine neue <code>TurnAction</code>.
	 * @param worldModel Referenz zum Weltmodell
	 * @param sender Referenz zum Sender
	 */
	public TurnAction(WorldModel worldModel, Sender sender)
	{
		super(worldModel, sender);
		resetHeadAngle = false;
	}

	/**
	 * Aktion fortsetzen
	 */
	public void resume()
	{
		Hero me = worldModel.getYourself();
		double istWinkel = me.getViewAngle() - me.getHeadAngle();
		double sollWinkel = destWinkel;
		double winkelAbw;
		double drehwinkel;
		
		winkelAbw = RoboCupMath.angleDiff(istWinkel, sollWinkel);
		
		// falls gewnscht, den Kopfwinkel zurcksetzen
		if (resetHeadAngle)
		{
			if (Math.abs(me.getHeadAngle()) > 2)
			{
				sender.turnNeck(-me.getHeadAngle());
			}
		}
	
		// Falls Winkelabweichung zu gro, Winkel korrigieren.		
		if (Math.abs(winkelAbw) > 5)
		{
			drehwinkel = winkelAbw;
			if (drehwinkel < 0)
			{
				drehwinkel = Math.max(drehwinkel, -160);
			}
			else
			{
				drehwinkel = Math.min(drehwinkel, 160);
			}

			// nur drehen, falls letzte Drehung schon etwas her				
			if (lastTurn < worldModel.getYourself().getTime() - 2)
			{
				sender.turn(drehwinkel);
				lastTurn = worldModel.getYourself().getTime();
			}
		}
		else
		{
			// Winkel stimmt etwa, also Aktion beenden
			actionFinished = true;
			actionPending = false;
		}
	}

	/**
	 * Zielwinkel festlegen.
	 * @param destWinkel der gewnschte Zielwinkel
	 */
	public void setDestWinkel(double destWinkel)
	{
		this.destWinkel = destWinkel;
	}
	
	/**
	 * Festlegen, ob der Kopfwinkel zurckgesetzt werden soll.
	 * @param resetHeadAngle Kopfwinkel zurcksetzen
	 */
	public void setResetHeadAngle(boolean resetHeadAngle)
	{
		this.resetHeadAngle = resetHeadAngle;
	}
}

