import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Visualisierungsfenster.
 * 
 * @author Jochen Rbenach
 * @version 1.12
 */
public class Visualization extends JFrame
{
	private PlayFieldPanel playField; //Referenz auf PlayFieldPanel
	private RoboCup[] roboCup; 
	private int curPlayerNum; //Nummer des anzuzeigenden Spielers
	private ButtonsPanel buttons; //Referenz
	private LabelsPanel labels; //Referenz
	private ScorePanel score; //Referenz

	/**
	 * Konstruktor
	 * 
	 * @param roboCup RoboCup[]
	 * @param playerNum int 
	 */
	public Visualization(RoboCup[] roboCup, int playerNum)
	{
		super("Visualisierung");

		this.roboCup = roboCup;
		curPlayerNum = playerNum;
		Container cp = getContentPane();
		buttons = new ButtonsPanel(this);
		
		//Wenn der Spieler existiert
		if (roboCup[playerNum] != null)
		{
			playField = new PlayFieldPanel(roboCup[playerNum].getWorldModel());
			labels = new LabelsPanel(roboCup[playerNum].getWorldModel());
			score = new ScorePanel(roboCup[playerNum].getWorldModel());
		}
		else
		{
			playField = new PlayFieldPanel(null);
			labels = new LabelsPanel(null);
			score = new ScorePanel(null);
		}
				
		cp.setLayout(new BorderLayout()); //Layout festlegen
		//fge ... cp hinzu
		cp.add(playField, BorderLayout.CENTER);
		cp.add(buttons, BorderLayout.WEST);
		cp.add(labels, BorderLayout.SOUTH);
		cp.add(score, BorderLayout.NORTH);
		
		//WindowListener zum Fenster schliessen
		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				System.exit(0);
			}

		});
		
		setSize(600, 450);	//gebe Gsse vor
		setVisible(true);	//mache Fenster sichtbar
	}
	
	/**
	 * updated playField, labels, score
	 */
	public void updateContent()
	{
		playField.repaint();
		labels.updateContent();
		score.updateContent();
	}

	/**
	 * Spieler an Visualisierung ab- und anhaengen
	 * 
	 * @param playerNum int
	 */
	public void visualizePlayer(int playerNum)
	{
		// Alten Spieler von der Visualisierung abhngen
		if (roboCup[curPlayerNum] != null)
		{
			roboCup[curPlayerNum].getReceiver().setVisualization(null);
		}
		
		if (roboCup[playerNum] != null)
		{
			playField.setWorldModel(roboCup[playerNum].getWorldModel());
			labels.setWorldModel(roboCup[playerNum].getWorldModel());
			score.setWorldModel(roboCup[playerNum].getWorldModel());
			// Neuen Spieler an die Visualisierung hngen
			roboCup[playerNum].getReceiver().setVisualization(this);
		}
		curPlayerNum = playerNum;
	}

	/**
	 * Returns the PlayerCount
	 * 
	 * @return int
	 */
	public int getPlayerCount()
	{
		return roboCup.length;
	}
	
	/**
	 * Returns the curPlayerNum
	 * 
	 * @return int
	 */
	public int getCurPlayerNum()
	{
		return curPlayerNum;
	}
}